# The LearningOnline Network with CAPA
# image click response style
#
# $Id: imageresponse.pm,v 1.42 2003/11/11 00:39:33 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
#FIXME LATER assumes multiple possible submissions but only one is possible 
#currently

package Apache::imageresponse;
use strict;
use Image::Magick;
use Apache::randomlylabel;
use Apache::Constants qw(:common :http);

BEGIN {
    &Apache::lonxml::register('Apache::imageresponse',('imageresponse'));
}

sub start_imageresponse {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    #when in a radiobutton response use these
    &Apache::lonxml::register('Apache::imageresponse',('foilgroup','foil','text','image','rectangle','conceptgroup'));
    push (@Apache::lonxml::namespace,'imageresponse');
    my $id = &Apache::response::start_response($parstack,$safeeval);
    if ($target eq 'meta') {
	$result=&Apache::response::meta_package_write('imageresponse');
    } elsif ($target eq 'analyze') {
	my $part_id="$Apache::inputtags::part.$id";
	push (@{ $Apache::lonhomework::analyze{"parts"} },$part_id);
    }
    return $result;
}

sub end_imageresponse {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    &Apache::response::end_response;
    pop @Apache::lonxml::namespace;
    &Apache::lonxml::deregister('Apache::imageresponse',('foilgroup','foil','text','image','rectangle','conceptgroup'));
    my $result;
    if ($target eq 'edit') { $result=&Apache::edit::end_table(); }
    return $result;
}

%Apache::response::foilgroup=();
sub start_foilgroup {
    %Apache::response::foilgroup=();
    $Apache::imageresponse::conceptgroup=0;
    &Apache::response::setrandomnumber();
    return '';
}

sub getfoilcounts {
    my ($parstack,$safeeval)=@_;

    my $max = &Apache::lonxml::get_param('max',$parstack,$safeeval,'-2');
    # +1 since instructors will count from 1
    my $count = $#{ $Apache::response::foilgroup{'names'} }+1;
    if (&Apache::response::showallfoils()) { $max=$count; }
    return ($count,$max);
}

sub whichfoils {
    my ($max)=@_;
    if (!defined(@{ $Apache::response::foilgroup{'names'} })) { return; }
    my @names = @{ $Apache::response::foilgroup{'names'} };
    my @whichopt =();
    while ((($#whichopt+1) < $max) && ($#names > -1)) {
	&Apache::lonxml::debug("Have $#whichopt max is $max");
	my $aopt;
	if (&Apache::response::showallfoils()) {
	    $aopt=0;
	} else {
	    $aopt=int(&Math::Random::random_uniform() * ($#names+1));
	}
	&Apache::lonxml::debug("From $#names elms, picking $aopt");
	$aopt=splice(@names,$aopt,1);
	&Apache::lonxml::debug("Picked $aopt");
	push (@whichopt,$aopt);
    }
    return @whichopt;
}

sub prep_image {
    my ($image,$mode,$name)=@_;
    my $part=$Apache::inputtags::part;
    my $respid=$Apache::inputtags::response['-1'];
    my $id=&Apache::loncommon::get_cgi_id();
    my %x;
    $x{"cgi.$id.BGIMG"}=$image;
    my ($x,$y)=split(/:/,$Apache::lonhomework::history{"resource.$part.$respid.submission"});
    #draws 2 xs on the image at the clicked location
    #one in white and then one in red on top of the one in white
    if (defined($x) && defined($y)) {
	$x{"cgi.$id.LINECOUNT"}=4;
	my $length = 6;
	my $width = 1;
	my $extrawidth = 2;
	$x{"cgi.$id.LINE0"}=
	    join(':',(($x-$length),($y-$length),($x+$length),($y+$length),
		      "FFFFFF",($width+$extrawidth)));
	$x{"cgi.$id.LINE1"}=
	    join(':',(($x-$length),($y+$length),($x+$length),($y-$length),
		      "FFFFFF",($width+$extrawidth)));
	$x{"cgi.$id.LINE2"}=
	    join(':',(($x-$length),($y-$length),($x+$length),($y+$length),
		      "FF0000",($width)));
	$x{"cgi.$id.LINE3"}=
	    join(':',(($x-$length),($y+$length),($x+$length),($y-$length),
		      "FF0000",($width)));
    }
    if ($mode eq 'answer') {
	my $width = 1;
	my $extrawidth = 2;
	my @areas = @{ $Apache::response::foilgroup{"$name.area"} };
	foreach my $area (@areas) {
	    my ($x1,$y1,$x2,$y2)=
		($area=~m/rectangle:\(([0-9]+),([0-9]+)\)\-\(([0-9]+),([0-9]+)\)/);
	    my $i=$x{"cgi.$id.BOXCOUNT"}++;
	    $x{"cgi.$id.BOX$i"}=join(':',($x1,$y1,$x2,$y2,"FFFFFF",
					  ($width+$extrawidth)));
	    $i=$x{"cgi.$id.BOXCOUNT"}++;
	    $x{"cgi.$id.BOX$i"}=join(':',($x1,$y1,$x2,$y2,"00FF00",$width));
	}
    }
    &Apache::lonnet::appenv(%x);
    return $id;
}

sub displayfoils {
    my ($target,@whichopt) = @_;
    my $result ='';
    my $name;
    my $temp=1;
    foreach $name (@whichopt) {
	$result.=$Apache::response::foilgroup{"$name.text"};
	&Apache::lonxml::debug("Text is $result");
	if ($target eq 'tex') {$result.="\\vskip 0 mm \n";} else {$result.="<br />\n";}
	my $image=$Apache::response::foilgroup{"$name.image"};
	&Apache::lonxml::debug("image is $image");
	if ( $target eq 'web' && $image !~ /^http:/ ) {
	    $image=&Apache::lonnet::filelocation($Apache::lonxml::pwd[-1],$image);
	    if (&Apache::lonnet::repcopy($image) ne OK) {
		$image='/home/httpd/html/adm/lonKaputt/lonlogo_broken.gif';
	    }
	}
	&Apache::lonxml::debug("image is $image");
	if ( &Apache::response::show_answer() ) {
	    if ($target eq 'tex') {
		$result.=$Apache::response::foilgroup{"$name.image"}."\\vskip 0 mm \n";
	    } else {
		my $token=&prep_image($image,'answer',$name);
		$result.="<img src=\"/adm/randomlabel.png?token=$token\" /><br />\n";
	    }
	} else {
	    if ($target eq 'tex') {
		$result.=$Apache::response::foilgroup{"$name.image"}."\\vskip 0 mm \n";
	    } else {
		my $id=$Apache::inputtags::response['-1'];
		my $token=&prep_image($image);
		my $temp=1;
		$result.="<input type=\"image\" name=\"HWVAL_$id:$temp\" ".
		    "src=\"/adm/randomlabel.png?token=$token\" /><br />\n";
	    }
	}
	$temp++;
    }
    return $result;
}

sub gradefoils {
    my (@whichopt) = @_;
    my $x;
    my $y;
    my $result;
    my $id=$Apache::inputtags::response['-1'];
    my $temp=1;
    foreach my $name (@whichopt) {
	$x=$ENV{"form.HWVAL_$id:$temp.x"};
	$y=$ENV{"form.HWVAL_$id:$temp.y"};
	&Apache::lonxml::debug("Got a x of $x and a y of $y for $name");
	if (defined($x) && defined($y) &&
	    defined(@{ $Apache::response::foilgroup{"$name.area"} })) {
	    my @areas = @{ $Apache::response::foilgroup{"$name.area"} };
	    my $grade="INCORRECT";
	    foreach my $area (@areas) {
		&Apache::lonxml::debug("Area is $area for $name");
		$area =~ m/([a-z]*):/;
		&Apache::lonxml::debug("Area of type $1");
		if ($1 eq 'rectangle') {
		    $grade=&grade_rectangle($area,$x,$y);
		} else {
		    &Apache::lonxml::error("Unknown area style $area");
		}
		&Apache::lonxml::debug("Area said $grade");
		if ($grade eq 'APPROX_ANS') { last; }
	    }
	    &Apache::lonxml::debug("Foil was $grade");
	    if ($grade eq 'INCORRECT') { $result= 'INCORRECT'; }
	    if (($grade eq 'APPROX_ANS') && ($result ne 'APPROX_ANS')) { $result=$grade; }
	    &Apache::lonxml::debug("Question is $result");
	    $temp++;
	}
    }
    $Apache::lonhomework::results{"resource.$Apache::inputtags::part.$id.submission"}="$x:$y";
    $Apache::lonhomework::results{"resource.$Apache::inputtags::part.$id.awarddetail"}=$result;
    return '';
}

sub end_foilgroup {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result='';
    my @whichopt;
    if ($target eq 'web' || $target eq 'grade' || $target eq 'tex' ||
	$target eq 'analyze') {
	my ($count,$max) = &getfoilcounts($parstack,$safeeval);
	if ($count>$max) { $count=$max }
	&Apache::lonxml::debug("Count is $count from $max");
	@whichopt = &whichfoils($max);
	if ($target eq 'web' || $target eq 'tex') {
	    $result=&displayfoils($target,@whichopt);
	} elsif ($target eq 'grade') {
	    if ( defined $ENV{'form.submitted'}) { &gradefoils(@whichopt); }
	} elsif ( $target eq 'analyze') {
	    &Apache::response::analyze_store_foilgroup(\@whichopt,
						      ['text','image','area']);
	}
    } elsif ($target eq 'edit') {
	$result=&Apache::edit::end_table();
    }
    return $result;
}

sub start_conceptgroup {
    $Apache::imageresponse::conceptgroup=1;
    %Apache::response::conceptgroup=();
    return '';
}

sub end_conceptgroup {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    $Apache::imageresponse::conceptgroup=0;
    my $result;
    if ($target eq 'web' || $target eq 'grade' || $target eq 'tex' ||
	$target eq 'analyze') {
	&Apache::response::pick_foil_for_concept($target,
						 ['area','text','image'],
						 \%Apache::hint::image,
						 $parstack,$safeeval);
    } elsif ($target eq 'edit') {
	$result=&Apache::edit::end_table();
    }
    return $result;
}

sub insert_foil {
    return '
       <foil>
           <image></image>
           <text></text>
           <rectangle></rectangle>
       </foil>
';
}

$Apache::imageresponse::curname='';
sub start_foil {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    if ($target eq 'web' || $target eq 'grade' || $target eq 'tex' ||
	$target eq 'analyze') {
	my $name = &Apache::lonxml::get_param('name',$parstack,$safeeval);
	if ($name eq '') { $name=$Apache::lonxml::curdepth; }
	if ( $Apache::imageresponse::conceptgroup
	     && !&Apache::response::showallfoils()) {
	    push(@{ $Apache::response::conceptgroup{'names'} }, $name);
	} else {
	    push(@{ $Apache::response::foilgroup{'names'} }, $name);
	}
	$Apache::imageresponse::curname=$name;
    }
    return '';
}

sub end_foil {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    if ($target eq 'edit') {
	$result=&Apache::edit::end_table();
    }
    return $result;
}

sub start_text {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result='';
    if ($target eq 'web' || $target eq 'tex' || $target eq 'analyze') { 
	&Apache::lonxml::startredirection; 
    } elsif ($target eq 'edit') {
	my $descr=&Apache::lonxml::get_all_text('/text',$parser);
	$result=&Apache::edit::tag_start($target,$token,'Task Description').
	    &Apache::edit::editfield($token->[1],$descr,'Text',60,2).
	    &Apache::edit::end_row();
    } elsif ($target eq "modified") {
	$result=$token->[4].&Apache::edit::modifiedfield('/text',$parser);
    }
    return $result;
}

sub end_text {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    if ($target eq 'web' || $target eq 'tex' || $target eq 'analyze') {
	my $name = $Apache::imageresponse::curname;
	if ( $Apache::imageresponse::conceptgroup
	     && !&Apache::response::showallfoils() ) {
	    $Apache::response::conceptgroup{"$name.text"} = &Apache::lonxml::endredirection;
	} else {
	    $Apache::response::foilgroup{"$name.text"} = &Apache::lonxml::endredirection;
	}
    } elsif ($target eq 'edit') {
	$result=&Apache::edit::end_table();
    }
    return $result;
}

sub start_image {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result='';
    if ($target eq 'web' || $target eq 'tex' || $target eq 'analyze') { 
	&Apache::lonxml::startredirection; 
    } elsif ($target eq 'edit') {
	my $bgimg=&Apache::lonxml::get_all_text('/image',$parser);
	$Apache::edit::bgimgsrc=$bgimg;
	$Apache::edit::bgimgsrcdepth=$Apache::lonxml::curdepth;

	$result=&Apache::edit::tag_start($target,$token,'Clickable Image').
	    &Apache::edit::editline($token->[1],$bgimg,'Image Source File',40);
	$result.=&Apache::edit::browse(undef,'textnode').' ';
	$result.=&Apache::edit::search(undef,'textnode').
	    &Apache::edit::end_row();
    } elsif ($target eq "modified") {
	$result=$token->[4].&Apache::edit::modifiedfield('/image',$parser);
    }
    return $result;
}

sub end_image {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    my $name = $Apache::imageresponse::curname;
    if ($target eq 'web') {
	my $image = &Apache::lonxml::endredirection;
	&Apache::lonxml::debug("original image is $image");
	if ( $Apache::imageresponse::conceptgroup
	     && !&Apache::response::showallfoils()) {
	    $Apache::response::conceptgroup{"$name.image"} = $image;
	} else {
	    $Apache::response::foilgroup{"$name.image"} = $image;
	}
    } elsif ($target eq 'analyze') {
	my $image = &Apache::lonxml::endredirection;
	if ( $Apache::imageresponse::conceptgroup
	     && !&Apache::response::showallfoils()) {
	    $Apache::response::conceptgroup{"$name.image"} = $image;
	} else {
	    $Apache::response::foilgroup{"$name.image"} = $image;
	}
    } elsif ($target eq 'edit') {
	$result=&Apache::edit::end_table();
    } elsif ($target eq 'tex') {
	my $src = &Apache::lonxml::endredirection;
	$src=&Apache::lonnet::filelocation($Apache::lonxml::pwd[-1],$src);
	my $width_param = '';
	my $height_param = '';
	my $scaling = .3;
	my $image = Image::Magick->new;
	my $current_figure = $image->Read($src);
	$width_param = $image->Get('width') * $scaling;;
	$height_param = $image->Get('height') * $scaling;;
	undef $image;
	my $epssrc = $src;
	$epssrc =~ s/(\.gif|\.jpg)$/\.eps/i;
	if (not -e $epssrc) {
	    my $localfile = $epssrc;
	    $localfile =~ s/.*(\/res)/$1/;	
	    my $file;
	    my $path;	
	    if ($localfile =~ m!(.*)/([^/]*)$!) {
		$file = $2;
		$path = $1.'/'; 
	    }	
	    my $signal_eps = 0;
	    my @content_directory = &Apache::lonnet::dirlist($path);
	    for (my $iy=0;$iy<=$#content_directory;$iy++) {
		my @tempo_array = split(/&/,$content_directory[$iy]);
		$content_directory[$iy] = $tempo_array[0];
		if ($file eq $tempo_array[0]) {
		    $signal_eps = 1;
		    last;
		}
	    }
	    if ($signal_eps) {
		my $eps_file = &Apache::lonnet::getfile($localfile);
	    } else {
		$localfile = $src;
		$localfile =~ s/.*(\/res)/$1/;	
		my $as = &Apache::lonnet::getfile($src);		      
	    }
	}
	my $file;
	my $path;	
	if ($src =~ m!(.*)/([^/]*)$!) {
	    $file = $2;
	    $path = $1.'/'; 
	}
	my $newsrc = $src;
	$newsrc =~ s/(\.gif|\.jpg|\.jpeg)$/\.eps/i;
	$file=~s/(\.gif|\.jpg|\.jpeg)$/\.eps/i;
	#do we have any specified size of the picture?
	my $TeXwidth = &Apache::lonxml::get_param('TeXwidth',$parstack,$safeeval); 
	my $TeXheight = &Apache::lonxml::get_param('TeXheight',$parstack,$safeeval); 
	my $width = &Apache::lonxml::get_param('width',$parstack,$safeeval);
	if ($TeXwidth ne '') { 
	    $width_param = $TeXwidth; 
	} elsif ($TeXheight ne '') { 
	    $width_param = $TeXheight/$height_param*$width_param;
	} elsif ($width ne '') {
	    $width_param = $width*$scaling;      
	}
	#where can we find the picture?
	if (-e $newsrc) {
	    if ($path) {
		$Apache::response::foilgroup{"$name.image"} ='\vskip 0 mm \noindent\graphicspath{{'.$path.'}}\includegraphics[width='.$width_param.' mm]{'.$file.'} ';
	    }
	} else {
	    my $temp_file;
	    my $filename = "/home/httpd/prtspool/$ENV{'user.name'}_$ENV{'user.domain'}_printout.dat";
	    $temp_file = Apache::File->new('>>'.$filename); 
	    print $temp_file "$src\n";
	    $Apache::response::foilgroup{"$name.image"} ='\vskip 0 mm \graphicspath{{/home/httpd/prtspool/}}\includegraphics[width='.$width_param.' mm]{'.$file.'} ';
	}
    } 
    return $result;
}

sub start_rectangle {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result='';
    if ($target eq 'web' || $target eq 'grade' || $target eq 'tex' ||
	$target eq 'analyze') { 
	&Apache::lonxml::startredirection; 
    } elsif ($target eq 'edit') {
	my $coords=&Apache::lonxml::get_all_text('/rectangle',$parser);
	$result=&Apache::edit::tag_start($target,$token,'Rectangle').
	    &Apache::edit::editline($token->[1],$coords,'Coordinate Pairs',40).
	    &Apache::edit::entercoordpair(undef,'textnode').
	    &Apache::edit::end_row();
    } elsif ($target eq "modified") {
	$result=$token->[4].&Apache::edit::modifiedfield('/rectangle',$parser);
    }
    return $result;
}

sub grade_rectangle {
    my ($spec,$x,$y) = @_;
    &Apache::lonxml::debug("Spec is $spec");
    $spec=~m/rectangle:\(([0-9]+),([0-9]+)\)\-\(([0-9]+),([0-9]+)\)/;
    my $x1=$1;
    my $y1=$2;
    my $x2=$3;
    my $y2=$4;
    &Apache::lonxml::debug("Point $x1,$y1,$x2,$y2");
    if ($x1 > $x2) { my $temp=$x1;$x1=$x2;$x2=$temp; }
    if ($y1 > $y2) { my $temp=$y1;$y1=$y2;$y2=$temp; }
    if (($x >= $x1) && ($x <= $x2) && ($y >= $y1) && ($y <= $y2)) {
	return 'APPROX_ANS';
    }
    return 'INCORRECT';
}

sub end_rectangle {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    if ($target eq 'web' || $target eq 'grade' || $target eq 'tex' ||
	$target eq 'analyze') {
	my $name = $Apache::imageresponse::curname;
	my $area = &Apache::lonxml::endredirection;
	&Apache::lonxml::debug("out is $area for $name");
	if ( $Apache::imageresponse::conceptgroup
	     && !&Apache::response::showallfoils()) {
	    push @{ $Apache::response::conceptgroup{"$name.area"} },"rectangle:$area";
	} else {
	    push @{ $Apache::response::foilgroup{"$name.area"} },"rectangle:$area";
	}
    } elsif ($target eq 'edit') {
	$result=&Apache::edit::end_table();
    }
    return $result;
}
1;
__END__
 
