# automated enrollment configuration handler
# $Id: lonpopulate.pm,v 1.6 2003/12/12 00:45:49 raeburn Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
package Apache::lonpopulate;

use strict;
use lib qw(/home/httpd/lib/perl);
use Apache::lonnet ();
use Apache::loncommon ();
use Apache::lonhtmlcommon ();
use Apache::lonlocal;
use Apache::Constants qw(:common :http REDIRECT);
use Time::Local;
use LONCAPA::Enrollment ();
use localenroll;

###############################################################
sub header {
    my $bodytag=&Apache::loncommon::bodytag('Classlist Manager');
    return(<<ENDHEAD);
<html>
<head>
<title>LON-CAPA Classlist Manager</title>
</head>
$bodytag
ENDHEAD
}

###############################################################

sub choose_header {
    my $bodytag=&Apache::loncommon::bodytag('Classlist Manager');
    my $scripttag = qq|
<script language='javascript' type='text/javascript'>
<!--
function process(calling) {
 var checker = 1
 var rad1 = 0
 var rad2 = 0
 var formName = document.forms.enter
 formName.action.value = calling
 if (calling == "chgsettings") {
   for (var j=0; j<formName.autoadds.length; j++) {
       if (formName.autoadds[j].checked) {
           rad1 = 1
       }
   }
   for (var k=0; k<formName.autodrops.length; k++) {
       if (formName.autodrops[k].checked) {
           rad2 = 1
       }
   }
   if (rad1 == 0) {
       alert("You must check either 'Enable' or 'Disable' for nightly additions based on classlist changes")
       checker = 0
   }
   if (rad2 == 0) {
       alert("You must check either 'Enable' or 'Disable' for nightly removals based on classlist changes")
       checker = 0
   }
 }
 if (calling == "updatenow") {
     var enrolldis
     var unenrolldis
     for (var j=0; j<formName.updateadds.length; j++) {
         if (formName.updateadds[j].value == 0) {
             enrolldis = j
         }
         if (formName.updateadds[j].checked) {
             rad1 = 1
         }
     }
     for (var k=0; k<formName.updatedrops.length; k++) {
         if (formName.updatedrops[k].value == 0) {
             unenrolldis = k 
         }
         if (formName.updatedrops[k].checked) {
             rad2 = 1
         }
     }
     if (rad1 == 0) {
         alert("You must check either 'Enable' or 'Disable' for nightly additions based on classlist changes")
         checker = 0
     }
     if (rad2 == 0) {
         alert("You must check either 'Enable' or 'Disable' for nightly removals based on classlist changes")
         checker = 0
     }
     if (formName.updatedrops[unenrolldis].checked && formName.updateadds[enrolldis].checked ) {
         alert("You have checked 'Disable' for both addition and removal of students\\n in the Registrar's classlist but not in your LON-CAPA course.  \\nHence there is no update to carry out")
         checker = 0
     }
 }
 if (checker == 1) {  
     formName.submit();
 }
}
// End hiding -->
</script>
|;
    return(<<ENDHEAD);
<html>
<head>
<title>LON-CAPA Classlist Manager</title>
$scripttag
$bodytag
ENDHEAD
}

sub print_mainbox {
    my ($r,$tasklongref,$realm,$reply) = @_;
    my $action = "information";
    if ( exists($ENV{'form.action'}) ) {
        $action = $ENV{'form.action'};
    }
    my $page = '';
    if ($action eq "information") {
        $page = "<b>Automated Enrollment</b>";
    } else {
        $page =  '<a href="/adm/populate">Automated Enrollment</a>';
        if ($reply) {
            if ($action eq "newcross") {
                $action = "crosslist";
            } elsif ($action eq "newsections") {
                $action = "sections"; 
            }
            $page .= "-&gt; <a href=\"/adm/populate?action=$action\">".$$tasklongref{$action}."</a> -&gt; <b>result</b>";
        } else {
            $page .=  " -&gt; <b>".$$tasklongref{$action}."</b>"; 
        }
    }
    $r->print(<<ENDTHIS);
<table width="100%" border="0" cellpadding="0" cellspacing="0">
 <tr>
  <td bgcolor="#CCCCFF"> 
   <font size="2"><a href="/adm/menu">$realm</a> -&gt; <a href="/adm/dropadd">Enrollment Manager</a> -&gt; $page</font><br/>
  </td>
  <td align="right" bgcolor="#CCCCFF" valign="top">
   <font size="+1">Automated Enrollment Manager&nbsp;</font>
  </td>
 </tr>
</table>
<table width="100%" border="0" cellpadding="0" cellspacing="0">
 <tr>
ENDTHIS
}

sub print_navmenu {
    my ($r,$tasksref,$tasklongref) = @_;
    my $action = "information";
    if (exists($ENV{'form.action'}) ) {
        $action = $ENV{'form.action'};
    }
    $r->print(<<ENDONE);
  <td width="10" valign="top" bgcolor="#DDFFFF">&nbsp;</td>
  <td width="20%" valign="top" bgcolor="#DDFFFF">
   <br/>
ENDONE
    foreach my $task (@{$tasksref}) {
        if ($task eq $action) {
            $r->print(" 
   <p>
   <font color=\"#999999\">
    <b>$$tasklongref{$task}</b><br/>
   </font>
   </p>"); 
        } else {
            $r->print("
   <p>
   <font color=\"#004263\">
    <b><a href=\"/adm/populate?action=$task\">$$tasklongref{$task}</a></b><br/>
   </font> 
   </p>");

        }
    }
    $r->print("
  <p>&nbsp;</p>
  </td>
  <td width=\"10\" valign=\"top\" bgcolor=\"#CCCCFF\">&nbsp;</td>
  <td width=\"10\" valign=\"top\" bgcolor=\"#FFFFFF\">&nbsp;</td>
  <td bgcolor=\"#ffffff\" valign=\"top\">");
}

###############################################################

sub print_main_frame {
  my ($r,$realm,$dom,$crs,$tasktitleref) = @_;
  my $action = "information";
  if (exists($ENV{'form.action'}) ) {
      $action = $ENV{'form.action'};
  }

# Get course settings
  my %enrollvar;
  my @bgcolors=("#eeeeee","#cccccc");
  my %settings = &Apache::lonnet::dump('environment',$dom,$crs);
  foreach my $item (keys %settings) {
      if ($item =~ m/^internal\.(.+)$/) {
          $enrollvar{$1} = $settings{$item};
      }
  }

  if ($action eq "information") {
      $r->print(<<ENDONE);
          <br/><table border='0' width='100%'>
            <tr>
              <td>&nbsp;</td>
              <td><b>Use the menu on the left to choose an enrollment management task.</b><br/><br/></td>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Automated adds/drops"</i> to enable or disable automatic nightly adds or drops in your LON-CAPA course based on instututional enrollment information.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Change enrollment dates"</i> to change the date of first automated enrollment and/or the date of last automated enrollment for registered students.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Notification of changes"</i> to enable or disable notification of enrollment changes and to add or remove course coordinators from the recipient list.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Change crosslisting"</i> to include or exclude enrollment from crosslisted classes.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Section settings"</i> to make changes to the choice of sections included for enrollment in your LON-CAPA course.</td>
            </tr>
              <td>&nbsp;</td>
              <td>Use <i>"Student photos"</i> to enable or disable automatic upload of student photos to your course.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Update roster now"</i> to add and/or drop students from your course based on the <b>most current</b> institutional classlist information.</td>
            </tr>
            <tr>
             <td colspan='2'>&nbsp;</td>
            </tr>
            <tr>
             <td>&nbsp;</td>
             <td><b>Note: if automated adds and/or drops are enabled, the nightly enrollment update will ONLY occur once the first enrollment date has been reached.</b></td>
            </tr>
          </table>
ENDONE
  } elsif ($action eq "chgsettings") {
      my @autosets = ("OFF","ON");
      $r->print(<<ENDTWO);
                  <form name="enter" method="post"><br/>
		  <table width="100%" border="0" cellpadding="2" cellspacing="2">
		   <tr>
		    <td align="left"><b>$$tasktitleref{$action}</b><br/>
		       Currently: Nightly adds: <i>$autosets[$enrollvar{autoadds}]</i>, Nightly drops: <i>$autosets[$enrollvar{autodrops}]</i>
		    </td>
                   </tr>
		  </table>
		  <table width="100%" border="0" cellpadding="3" cellspacing="3">
		    <tr>
		     <td>
		         Additions based on classlist changes:&nbsp;&nbsp;
ENDTWO
      if ($enrollvar{autoadds}) {
          $r->print("
			    <input type=\"radio\" name=\"autoadds\" value=\"1\" checked=\"true\" />&nbsp;Enable&nbsp;&nbsp;&nbsp;
			    <input type=\"radio\" name=\"autoadds\" value=\"0\" />&nbsp;Disable
         ");
      } else {
          $r->print("
                            <input type=\"radio\" name=\"autoadds\" value=\"1\" />&nbsp;Enable&nbsp;&nbsp;&nbsp;
                            <input type=\"radio\" name=\"autoadds\" value=\"0\" checked=\"true\" />&nbsp;Disable
         ");
      }
      $r->print("
              </td>
             </tr>
             <tr>
              <td>
 	       Removals based on classlist changes:&nbsp;&nbsp;");
      if ($enrollvar{autodrops}) {
          $r->print("
                <input type=\"radio\" name=\"autodrops\" value=\"1\" checked=\"true\" />&nbsp;Enable&nbsp;&nbsp;&nbsp;
                <input type=\"radio\" name=\"autodrops\" value=\"0\" />&nbsp;Disable");
      } else {
          $r->print("
                <input type=\"radio\" name=\"autodrops\" value=\"1\" />&nbsp;Enable&nbsp;&nbsp;&nbsp;
                <input type=\"radio\" name=\"autodrops\" value=\"0\" checked=\"true\" />&nbsp;Disable");
      }
      $r->print("
              </td>
             </tr>
             <tr>
              <td>
               <font color=\"#888888\">
Note: Any students added manually by course coordinators using the Enrollment Manager will be unaffected by the nightly removal process if you choose to enable it.
               </font>
              </td>
             </tr>
             <tr>
              <td align=\"right\">
               <input type=\"button\" name=\"chgsettings\" value=\"Go\" onClick=\"process('chgsettings')\" />
	      </td>
             </tr>
	    </table>
            <input type=\"hidden\" name=\"action\" value=\"$action\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form>
      ");
  } elsif ($action eq "setdates") {
      my ($start_table,$end_table) = &date_setting_table($enrollvar{autostart},$enrollvar{autoend});
      my $oldstartshow = localtime($enrollvar{autostart});
      my $oldendshow = localtime($enrollvar{autoend});
      if ($enrollvar{autoend} == 0) {
          $oldendshow = "No ending date";
      }
      $r->print(<<ENDTWO);
                  <form name="enter" method="post"><br/>              
                  <table width="100%" border="0" cellpadding="2" cellspacing="2">
	           <tr>
	            <td align="left"><b>$$tasktitleref{$action}</b><br/>
	             Currently: First enrollment: <i>$oldstartshow</i>, Last enrollment: <i>$oldendshow</i>
	            </td>
                   </tr>
	          </table>
                  <table width="100%" border="0" cellpadding="3" cellspacing="3">
                   <tr>
                    <td align="left" colspan="2">
                     <table border="0" cellspacing="0" cellpadding="2">
                      <tr>
                       <td colspan="3">
                        <i>Set date of first automated enrollment for registered students</i>
                       </td>
                      </tr>
                      <tr>
                       <td>$start_table
                       </td>
                      </tr>
                     </table>
                    </td>
                   </tr>
                   <tr>
                    <td colspan="2"><font color="#888888">If automated adds and/or drops are enabled, then your class roster will be automatically updated nightly, once the first enrollment date has been reached. Prior to this date, the class roster will only contain students you have added directly using the standard LON-CAPA enrollment tools</font></td>
                   </tr>
                   <tr>
                    <td align="left" colspan="2">
                     <table border="0' cellspacing="0" cellpadding="2">
                      <tr>
                       <td colspan="3">
                        <i>Set date of last automated enrollment for registered students</i>
                       </td>
                      </tr>
                      <tr>
                       <td>$end_table
                       </td>
                      </tr>
                     </table>
                    </td>
                   </tr>
                   <tr>
                    <td colspan="2"><font color="#888888">If automated adds and/or drops are enabled, then your class roster will be automatically updated nightly, until the last enrollment date has been reached.</font></td>
                   </tr>
                  </table>
                  <table width="100%">
                   <tr>
                    <td align="right">
                      <input type="button" name="setdates" value="Go" onClick="process('setdates')" />
                    </td>
	           </tr>
                  </table>
                  <input type=\"hidden\" name=\"action\" value=\"$action\">
                  <input type=\"hidden\" name=\"state\" value=\"process\">
                  </form>
ENDTWO
  } elsif ($action eq "notify") {
      my @notified = ();
      if ($enrollvar{notifylist} =~ m/,/) {
          @notified = split/,/,$enrollvar{notifylist};
      } else {
          @notified = $enrollvar{notifylist};
      }
      my $notifycount = 0;
      foreach  (@notified) {
        unless ($_ eq '') { $notifycount ++; } 
      }
      my $noteset = '';
      if ($notifycount) {
          $noteset = "ON";
      } else {
          $noteset = "OFF";
      }
    
      $r->print("
                  <form name=\"enter\" method=\"post\"><br/>
                   <table width=\"100%\" border=\"0\" cellpadding=\"6\" cellspacing=\"0\">
                    <tr>
                     <td align=\"left\"><b>$$tasktitleref{$action}</b><br/>
                      Currently: Notification: $noteset
                     </td>
                    </tr>
                   </table>
                   <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
                    <tr>
                     <td>
                      Notification of LON-CAPA course roster changes resulting from nightly automated enrollment process?
      ");
      if ($notifycount) {
          $r->print("
                        <input type=\"radio\" name=\"notify\" value=\"1\" checked=\"true\" />&nbsp;Yes&nbsp;&nbsp;&nbsp;
                        <input type=\"radio\" name=\"notify\" value=\"0\" />&nbsp;No
          ");
      } else {
          $r->print("
                        <input type=\"radio\" name=\"notify\" value=\"1\" />&nbsp;Yes&nbsp;&nbsp;&nbsp;
                        <input type=\"radio\" name=\"notify\" value=\"0\" checked=\"true\" />&nbsp;No
          ");
      }
      $r->print("
              </td>
             </tr>
      ");
      my $option_str = '';
      my $lastpname = '';
      my $option_count = 0;

      my %coursepersonnel=&Apache::lonnet::get_course_adv_roles($dom.'/'.$crs);
      foreach (sort keys %coursepersonnel) {
          if ($_ eq &mt("Course Coordinator") )
          {
              my @ccs = ();
              if ($coursepersonnel{$_} =~ m/,/) {
                  @ccs = split(/,/,$coursepersonnel{$_});
              } else {
                  @ccs = $coursepersonnel{$_};
              }
              foreach (@ccs) {
                  my ($puname,$pudom)=split(/\:/,$_);
                  my $pname = &Apache::loncommon::plainname($puname,$pudom);
                  if (grep/^$puname\@$pudom$/,@notified) {
                      $option_str .="<option value=\"$puname\@$pudom\" selected>$pname</option>\n";
                  } else {
                      $option_str .="<option value=\"$puname\@$pudom\">$pname</option>\n";
                  }
                  $option_count ++;
                  $lastpname = $pname.'@'.$pudom;
              }
          }
      }
      if ($option_count > 1) {
          my $optionsize = $option_count;
          if ($optionsize > 6) {
              $optionsize = 6;
          } 
          $r->print("
             <tr>
              <td>
Select which course coordinators should receive information about roster changes whenever any adds or drops occur during the nightly enrollment check.<br/><br/>
                <select name=\"notifylist\" multiple size=\"$optionsize\">$option_str</select>
              </td>
             </tr>
             <tr>
              <td>
               Select more than one recipient by using the following keyboard and mouse combinations:<br/>
             <ul>
              <li>Apple-click: MacOS</li>
              <li>Ctrl-click: Windows</li>
              <li>Left click: Linux</li>
             </ul>
            </td>
           </tr> 
          ");
      } else { 
          $r->print("
           <tr>
            <td>
As you are the only user with the role of course coordinator in this course, if you choose to enable notification of roster changes, you will be the sole recipient of LON-CAPA messages generated whenever any adds or drops occur during the nightly enrollment check.
              <input type=\"hidden\" name=\"notifylist\" value=\"$lastpname\" />
            </td>
           </tr>
          ");
      }
      $r->print("
             <tr>
              <td align=\"right\">
               <input type=\"button\" name=\"notifyset\" value=\"Go\" onClick=\"process('notify')\" />
              </td>
             </tr>
            </table>
            <input type=\"hidden\" name=\"action\" value=\"$action\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form>
      ");
  } elsif ($action eq "crosslist") {
      my @xlists = ();
      if ($enrollvar{crosslistings} =~ m/,/) {
          @xlists = split/,/,$enrollvar{crosslistings};
      } elsif ($enrollvar{crosslistings} ne '') {
          $xlists[0] = $enrollvar{crosslistings};
      }
      my $cross_str = @xlists;
      $r->print("
            <form name=\"enter\" method=\"post\"><br/>
            <table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
             <tr>
              <td align=\"left\"><b>$$tasktitleref{$action}</b><br/>
      ");
      if ($cross_str > 0) {
          $r->print("
                Currently, LON-CAPA course: $realm ($enrollvar{coursecode}) is crosslisted with $cross_str course section(s).  Students enrolling in these course sections will be automatically added to the class roster for the course, if you have chosen to enable a nightly automated enrollment update.  For each crosslisting, leave the checkbox checked if you want registered students in that course to be included in the student roster for LON-CAPA course: <b>$realm ($enrollvar{coursecode})</b>; otherwise uncheck it. If you wish to change the section/group ID assigned in your LON-CAPA course for a crosslisted course, enter the new section/group ID in the appropriate textbox. The LON-CAPA section/group ID can be left (or set to) empty, if you do not wish to tie a section/group ID to this crosslisting. If you wish to add new crosslisted courses, enter the number of new courses to add in the textbox at the bottom of the page. You will provide information about each of the new crosslistings on a subsequent page.  Click 'Go' to store your changes.
              </td>
             </tr>
            </table>
            <br/>
            <table border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td>
               <table border=\"0\" width=\"100%\" cellspacing=\"3\" cellpadding=\"3\">
                <tr bgcolor=\"#CCCCFF\" align=\"left\">
                 <td><b>Enrollment?</b></td>
                 <td><b>Crosslisted course</b></td>
                 <td><b>LON-CAPA section/groupID</b></td>
                </tr>
          ");
          for (my $i=0; $i<@xlists; $i++) {
              my $xl = '&nbsp;';
              my $gp = '&nbsp;';
              if ($xlists[$i] =~ /(\w+):(.+)$/) {
                  $xl = $1;
                  $gp = $2;
              }               
              my $colflag = $i%2;
              $r->print("
                <tr bgcolor=\"$bgcolors[$colflag]\" align=\"left\">
                 <td><input type=\"checkbox\" name=\"cross_$i\" checked=\"true\"/></td>
                 <td>$xl</td>
                 <td><input type =\"text\" size=\"10\" name=\"gp_$i\" value=\"$gp\"></td>
                </tr> 
              ");
          }
          $r->print("  </table>
              </td>
             </tr>
            </table>
          ");
      }
      else {
          $r->print("
                Currently no crosslisted courses are recorded for $enrollvar{coursecode}.
              </td>
             </tr>
            </table>
          ");
      }
      $r->print("   
            <br/>
            <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td align=\"left\">
               <b>Add new crosslistings.</b><br/>
               Number of new crosslistings to add:&nbsp;&nbsp;<input type=\"text\" size=\"2\" name=\"numcross\" value=\"0\" />
              </td>
             </tr>
            </table>
            <br/>
            <table width=\"100%\" border=\"0\" cellspadding=\"2\" cellspacing=\"2\">
             <tr>
              <td align=\"right\">
               <input type=\"button\" name=\"crosslist\" value=\"Go\" onClick=\"process('crosslist')\" />
              </td>
             </tr>
            </table>
            <input type=\"hidden\" name=\"action\" value=\"$action\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form> 
      ");
  } elsif ($action eq "sections") {
      my @sections = &localenroll::get_sections($enrollvar{coursecode});
      my @storedsections = ();
      my @currsections = ();
      my %sec_id = ();
      if ($enrollvar{sectionnums} =~ m/,/) {
          @storedsections = split/,/,$enrollvar{sectionnums};
      } else {
          $storedsections[0] = $enrollvar{sectionnums};
      }
      foreach (@storedsections) {
          if ($_ =~ m/^(\w+):(\w*)$/) {
              push @currsections, $1;
              $sec_id{$1} = $2;
          }
      }
      if (@sections > 0) {
          if (@sections > 1) {
              my $secshow = @sections;
              $r->print("
            <form name=\"enter\" method=\"post\"><br/>
            <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td align=\"left\">
               <b>$$tasktitleref{$action}</b><br/>
               Your institution's course catalog includes $secshow sections for course code: $enrollvar{coursecode}.
              </td>
             </tr>
             <tr>
              <td>For each section, check the checkbox if you want registered students in that section to be included in the student roster for LON-CAPA course: <b>$realm ($enrollvar{coursecode})</b>; otherwise uncheck it. If you want to change the section/group designation used for this section in LON-CAPA, delete the current value in the LON-CAPA section/group textbox and enter the new value. The LON-CAPA section/group ID can be left (or set to) empty, if you do not wish to tie a section/group ID to this section. To add a new section, check the 'Enrollment in this course?' checkbox, and enter the desired LON-CAPA section/groupID in the appropriate textbox. Click 'Go' to store
your changes.</td>
             </tr>
            </table>
            <br/>
            <table border=\"0\" width=\"100%\" cellspacing=\"3\" cellpadding=\"3\">
             <tr>
              <td>
               <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
                <tr bgcolor=\"#CCCCFF\">
                 <td><b>Section</b></td>
                 <td><b>Current status</b></td>              
                 <td><b>LON-CAPA section/group ID</b></td>
                 <td><b>Enrollment in this course?</b></td>
                </tr>
              ");
              for (my $i=0; $i<@sections; $i++) {
                  my $colflag = $i%2;
                  my $shrflag = 0;
                  $r->print("<tr bgcolor=\"$bgcolors[$colflag]\" align=\"left\">
                   <td>$sections[$i]<input type=\"hidden\" name=\"secnum_$i\" value=\"$sections[$i]\" /></td>
                   <td>
                  ");
                  if (grep/^$sections[$i]$/,@currsections) {
                      $r->print("Enrollment active");
                      $shrflag = 1;
                  } else {
                      $r->print("Enrollment inactive");
                  }
                  if ($shrflag) {
                      $r->print("</td><td><input type=\"text\" name=\"loncapasec_$i\" value=\"$sec_id{$sections[$i]}\"></td><td><input type=\"checkbox\" name=\"sec_$i\" checked=\"true\" /></td></tr>"); 
                  } else {
                      $r->print("</td><td><input type=\"text\" name=\"loncapasec_$i\" value=\"\"><td><input type=\"checkbox\" name=\"sec_$i\" /></td></tr>");
                  }
              }
              $r->print("
                  </tr>
                 </table>
                </td>
               </tr>
              </table>
              <br/>
              <table width=\"100%\" border=\"0\" cellspacing=\"3\" cellpadding=\"3\">
               <tr> 
                <td align=\"right\">
                 <input type=\"hidden\" name=\"secshow\" value=\"$secshow\" />
                 <input type=\"button\" name=\"sections\" value=\"Go\" onClick=\"process('sections')\" />
                </td>
               </tr>
              </table>
              <input type=\"hidden\" name=\"action\" value=\"$action\">
              <input type=\"hidden\" name=\"state\" value=\"process\">
              </form>
              ");
          } else {
              $r->print("<br/>            
            <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td align=\"left\">
               <b>$$tasktitleref{$action}</b><br/>
              </td>
             </tr>
            </table>
            <table border=\"0\" cellspacing=\"2\" cellpadding=\"2\">
             <tr>
              <td>As your institution's course catalog only includes a single section for this course, there are no additional sections that can be set for inclusion in the LON-CAPA roster for the course.</td>
            </tr>
           </table>
              ");
          } 
      } else {
          $r->print("
            <form name=\"enter\" method=\"post\"><br/>
            <table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
             <tr>
              <td align=\"left\"><b>$$tasktitleref{$action}</b><br/>
          ");
          if (@currsections) {
              my $secshow = @currsections;
              $r->print("
                Currently, LON-CAPA course: $realm ($enrollvar{coursecode}) incorporates enrollment from $secshow sections.  Students enrolling in any of these course sections will be automatically added to the class roster for the course, if you have chosen to enable a nightly automated enrollment update. For each section, uncheck the checkbox if you want registered students in that section to cease being included in the student roster for LON-CAPA course: <b>$realm ($enrollvar{coursecode})</b>; otherwise leave it checked. If you want to change the section/group designation used for this section in LON-CAPA, delete the current value in the LON-CAPA section/groupID textbox and enter the new value.  If you wish to add new course section, enter the number of new sections to add in the textbox at the bottom of the page. You will provide information about each of the new sections on a subsequent page. Click 'Go' to store your changes.
              </td>
             </tr>
            </table>
            <br/>
            <table border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td>
               <table border=\"0\" width=\"100%\" cellspacing=\"3\" cellpadding=\"3\">
                <tr bgcolor=\"#CCCCFF\">
                 <td><b>Enrollment?</b></td>
                 <td><b>Section</b></td>
                 <td><b>LON-CAPA section/groupID</b></td>
                </tr>
              ");
              for (my $j=0; $j<@currsections; $j++) {
                  my $colflag = $j%2;
                  $r->print("
                <tr bgcolor=\"$bgcolors[$colflag]\" align=\"left\">
                 <td><input type=\"checkbox\" name=\"sec_$j\" checked=\"true\"/></td>
                 <td>$currsections[$j]</td>
                 <td><input type=\"text\" name=\"secgp_$j\" size=\"10\" value=\"$sec_id{$currsections[$j]}\"></td>
                </tr> 
                  ");
              }
              $r->print("  </table>
              </td>
             </tr>
            </table>
              ");
          } else {
              $r->print("
                Currently no sections of $realm ($enrollvar{coursecode}) are contributing enrollment to the LON-CAPA class roster.
              </td>
             </tr>
            </table>
              ");
          }
          $r->print("   
            <br/>
            <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td align=\"left\">
               <b>Add enrollment from additional sections.</b><br/>
               Number of new sections to add:&nbsp;&nbsp;<input type=\"text\" size=\"2\" name=\"numsec\" value=\"0\" />
              </td>
             </tr>
            </table>
            <br/>
            <table width=\"100%\" border=\"0\" cellspadding=\"2\" cellspacing=\"2\">
             <tr>
              <td align=\"right\">
               <input type=\"button\" name=\"sections\" value=\"Go\" onClick=\"process('sections')\" />
              </td>
             </tr>
            </table>
            <input type=\"hidden\" name=\"action\" value=\"$action\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form> 
          ");
      }
  } elsif ($action eq "photos") {
      my @photosets = ("OFF","ON");
      $r->print("
                  <form name=\"enter\" method=\"post\"><br/>
                  <table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
                   <tr>
                    <td align=\"left\"><b>$$tasktitleref{$action}</b><br/>
                       Currently: Student photo import:  <i>$photosets[$enrollvar{showphotos}]</i>
                    </td>
                   </tr>
                  </table>
                  <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
                    <tr>
                     <td>
                         Automatic import of student photos from institutional data repository:&nbsp;&nbsp;

      ");
      if ($enrollvar{showphotos}) {
          $r->print("
                        <input type=\"radio\" name=\"showphotos\" value=\"1\" checked=\"true\" />&nbsp;Yes&nbsp;&nbsp;&nbsp;
                        <input type=\"radio\" name=\"showphotos\" value=\"0\" />&nbsp;No
          ");
      } else {
          $r->print("
                        <input type=\"radio\" name=\"showphotos\" value=\"1\" />&nbsp;Yes&nbsp;&nbsp;&nbsp;
                        <input type=\"radio\" name=\"showphotos\" value=\"0\" checked=\"true\" />&nbsp;No
          ");
      }
      $r->print("
                     </td>
                    </tr>
                    <tr>
                     <td>
                      <font color=\"#888888\">
Note: if you enable automatic import of student photos, your course will automatically have access to photos stored by your institution for all officially registered students, via a conduit established by your LON-CAPA domain coordinator.  
                      </font>
                     </td>
                    </tr>
                    <tr>
                     </td>&nbsp;</td>
                    </tr>
                    <tr>
                     <td align=\"right\">
                      <input type=\"button\" name=\"showphotos\" value=\"Go\" onClick=\"process('photos')\">
                     </td>
                    </tr>
                   </table>
                   <input type=\"hidden\" name=\"action\" value=\"$action\">
                   <input type=\"hidden\" name=\"state\" value=\"process\">
                   </form>
      ");
  } elsif ($action eq "updatenow") {
      $r->print("
                   <form name=\"enter\" method=\"post\"><br/>
		   <table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
		    <tr>
		     <td align=\"left\"><b>$$tasktitleref{$action}</b>
		     </td>
                    </tr>
		   </table>
		   <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
		    <tr>
		     <td>
	              Add any students currently included in institutional classlist(s) but not enrolled in your LON-CAPA course. 
		      <input type=\"radio\" name=\"updateadds\" value=\"1\">&nbsp;Yes&nbsp;
		      <input type=\"radio\" name=\"updateadds\" value=\"0\">&nbsp;No&nbsp;
                     </td>
                    </tr>
                    <tr>
                     <td>
 	              Expire students previously added by nightly enrollment process, but no longer listed in institutional  classlist(s).                  
                <input type=\"radio\" name=\"updatedrops\" value=\"1\">&nbsp;Yes&nbsp;
                <input type=\"radio\" name=\"updatedrops\" value=\"0\">&nbsp;No&nbsp;<br/>
              </td>
             </tr>
             <tr>
              <td>
               <font color=\"#888888\">
Note: Any students previously added manually by course coordinator(s) using either 'Upload classlist CSV file' or 'Enroll a single user' will be unaffected by the removal process.
               </font>
              </td>
             </tr>
             <tr>
              <td align=\"right\">
               <input type=\"button\" name=\"updatenow\" value=\"Go\" onClick=\"process('updatenow')\">
              </td>
             </tr>
	    </table>
            <input type=\"hidden\" name=\"action\" value=\"$action\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form>
      ");
  }
}

###############################################################
sub print_doc_base {
  my $r = shift;
  $r->print(<<ENDBASE);
  </td>
 </tr>
</table>
<br/>
</body>
</html> 
ENDBASE
}
 
###################################################################
sub print_chgsettings_response {
  my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
  my %settings = &Apache::lonnet::get('environment',['internal.autoadds','internal.autodrops'],$dom,$crs);
  my $curradds = $settings{'internal.autoadds'};
  my $currdrops = $settings{'internal.autodrops'};
  my $autoadds = '';
  my $autodrops = '';

  if ( exists($ENV{'form.autoadds'}) ) {
      $autoadds=$ENV{'form.autoadds'};
  }
  if ( exists($ENV{'form.autodrops'}) ) {
      $autodrops=$ENV{'form.autodrops'};
  }

  my $response = "";
  my %cenv = ('internal.autoadds' => $autoadds,
              'internal.autodrops' => $autodrops);
  my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
  if ($reply !~ /^ok$/) {
       $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
  } else {
      if ($autoadds) {
          if ($curradds) {
              $response = "Nightly additions based on classlist changes still <b>enabled</b><br/>";
          } else {
              $response = "Nightly additions based on classlist changes now <b>enabled</b><br/>";
          } 
      } else {
          if ($curradds) {
              $response = "Nightly additions based on classlist changes now <b>disabled</b><br/>";
          } else {
              $response = "Nightly additions based on classlist changes still <b>disabled</b><br/>";
          }
      }
      if ($autodrops) {
          if ($currdrops) {
              $response .= "Nightly removals based on classlist changes still <b>enabled</b><br>";
          } else {
              $response .= "Nightly removals based on classlist changes now <b>enabled</b><br/>";
          }
      } else {
          if ($currdrops) {
              $response .= "Nightly removals based on classlist changes now <b>disabled</b><br>";
          } else {
              $response .= "Nightly removals based on classlist changes still <b>disabled</b>";
          }
      }
  }
  &print_reply($r,$response,$$tasktitleref{$action});
  return;
}

sub print_setdates_response {
  my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
  my %settings = &Apache::lonnet::get('environment',['internal.autostart','internal.autoend'],$dom,$crs);
  my $currstart = $settings{'internal.autostart'};
  my $currend = $settings{'internal.autoend'};
  my $response = '';
  my ($autostart,$autoend) = &get_dates_from_form();
  my $showstart = localtime($autostart);
  my $showend = '';
  if ($autoend) {
      $showend = localtime($autoend);
  } else {
      $showend = "'No end date'";
  } 

  my %cenv = ('internal.autostart' => $autostart,
              'internal.autoend' => $autoend);
  my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
  if ($reply !~ /^ok$/) {
       $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
  } else {
      if ($currstart == $autostart) {
          $response = "The first date for automated enrollment has been left unchanged as $showstart.<br/>";
      } else {
          $response = "The first date for automated enrollment has been changed to $showstart.<br/>";
      } 
      if ($currend == $autoend) {
          $response .= "The last date for automated enrollment has been left unchanged as $showend.<br/>";
      } else {
          $response .= "The last date for enrollment has been left unchanged as $showend.<br/>";
      }
 
# Generate message in case where old first access date was later than today, but new first access date is now today or earlier.

      my $rosterupdated = 0;
      my $firstaccess = "";
      my $nextupdate = "";
      my $lastupdate = "";

      my $nowstamp = time;
      my @date_list=localtime(time);
      my $cur_year = $date_list[5];
      my $curday = $date_list[3];
      my $curmonth = $date_list[4];
      my $lastmidnt = timelocal(0,0,0,$date_list[3],$date_list[4],$date_list[5]);
      my $nextmidnt = 86400 + $lastmidnt;

      my $todayupdate = timelocal(0,40,4,$date_list[3],$date_list[4],$date_list[5]);
      my $lastupdate = $todayupdate - 86400;
      if ($nowstamp < $todayupdate) {
          $nextupdate = "today";
      } else {
          $nextupdate = "tomorrow";
      }
      if ($currstart < $lastupdate) {
          $rosterupdated = 1;
      }
      if ($autostart < $nextmidnt ) {
          if ( $autostart >= $lastmidnt) {
              $firstaccess = "today";
          } else {
              $firstaccess = "a date prior to today";
          }
          if (($nowstamp >= $autostart) && ($rosterupdated == 0)) {
              $response .= qq|<br>Although you have now set the first enrollment date to $firstaccess, automatic enrollment will <b>not</b> occur until the next automatic enrollment update occurs for all LON-CAPA courses at 4.40 am $nextupdate. If you wish to grant immediate course access for registered students included in the institutional classlist for this class, please visit the <a href="/adm/populate?action=updatenow">roster update page</a>.<br>|;
          }
      }
  }
  &print_reply($r,$response,$$tasktitleref{$action});
  return;
}

sub print_notify_response {
  my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;

# Get current settings
  my %settings = &Apache::lonnet::get('environment',['internal.notifylist','internal.coursecode'],$dom,$crs);
  my $notifylist = $settings{'internal.notifylist'};
  my $coursecode = $settings{'internal.coursecode'};
  my @currpeople = ();
  my $notify = 0;
  my @people = ();

  if ($notifylist =~ m/,/) {
      @currpeople = split/,/,$notifylist;
  } else {
      @currpeople = $notifylist;
  }
  my $currcount = 0;
  foreach  (@currpeople) {
      unless ($_ eq '') { $currcount ++; } 
  }

  if ( exists($ENV{'form.notify'}) ) {
      $notify=$ENV{'form.notify'};
  } 
  if ( exists($ENV{'form.notifylist'}) ) {
        if (ref($ENV{'form.notifylist'})) {
            @people=@{ $ENV{'form.notifylist'} };
        } else {
            $people[0]=$ENV{'form.notifylist'};
        }
  }
  my $response = "";
  my $peoplestr = "";
  if ($notify) { $peoplestr = join(",",@people); }
  my %cenv = ('internal.notifylist' => $peoplestr);
  my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
  if ($reply !~ /^ok$/) {
       $response = "There was a problem processing your requested changes. The notification settings for this course have been left unchanged.<br/>";  
  } else {
      if ($notify) {
          if (@people) { 
              if ($currcount) {
                  $response = "Notification of enrollment changes still <b>enabled</b><br/>";
              } else {
                  $response = "Notification of enrollment changes now <b>enabled</b><br/>";
              }
              $response .= "<br/>The following will receive notification if there are any changes in enrollment in LON-CAPA course: $realm ($coursecode) as a result of
the nightly enrollment check: <br/><ul>\n";
              foreach my $person (@people) {
                  $response .= "<li>$person</li>\n";
              }
              $response .= "</ul>\n";
          } else {
              $response = "Notification of enrollment changes was <b> not enabled</b> as no course coordinators were selected as recipients.<br/>";
          }
      } else {
          if ($currcount) { 
              $response = "Notification of enrollment changes now <b>disabled</b><br/>";
          } else {
              $response = "Notification of enrollment changes still <b>disabled</b><br/>";
          }
      }
  }
  &print_reply($r,$response,$$tasktitleref{$action});
  return;
}

sub print_crosslistings_menu () {
  my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
  my %settings = &Apache::lonnet::get('environment',['internal.crosslistings','internal.coursecode'],$dom,$crs);
  my @currxlists = ();
  my @xlists = ();
  my $crosscount = 0;
  my $removecount = 0;
  my $xliststr = '';
  my $response = '';
  my $coursecode = $settings{'internal.coursecode'};
  if ($settings{'internal.crosslistings'} =~ m/,/) {
      @currxlists = split/,/,$settings{'internal.crosslistings'};
  } elsif ($settings{'internal.crosslistings'} ne '') {
      $currxlists[0] = $settings{'internal.crosslistings'};
  }

  if (@currxlists > 0) {
      for (my $i=0; $i<@currxlists; $i++) {
          my $xlist = "cross_".$i;
          my $gp = "gp_".$i;
          if ( exists($ENV{"form.$xlist"}) ) {
              my $xlistentry = '';
              if ($currxlists[$i] =~ m/^(\w+:)/) {
                  $xlistentry = $1;
              }
              if ( exists($ENV{"form.$gp"}) ) {
                  $xlistentry .= $ENV{"form.$gp"};
              }
              push @xlists,$xlistentry;
              $crosscount ++;
          } else {
              $removecount ++;
          }
      }
  }

  if ($crosscount > 1) {
      $xliststr = join(",",@xlists);
  } else {
      $xliststr = $xlists[0];
  }
  my %cenv = ('internal.crosslistings' => $xliststr);
  my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
  if ($reply !~ /^ok$/) {
      $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
  } else {
      if ($removecount > 0) {
          $response = "A total of $removecount courses are no longer crosslisted with LON-CAPA course: $realm ($coursecode).<br/><br/>";
      }
      if ($crosscount > 0) {
          $response .=  "The $crosscount courses listed below remain crosslisted with this LON-CAPA course, and students enrolling in these course sections will be automatically added to the class roster for the course, if you have chosen to enable a nightly automated enrollment update.<br/><ul>\n";
          foreach (@xlists) {
              my ($xlist,$gp) = split/:/,$_;
              $response .= "<li>$xlist - ID: $gp</li>\n";
          }
          $response .= "</ul><br/>\n";
      }
  }
  if ( exists($ENV{'form.numcross'}) ) {
      my $numcross = $ENV{'form.numcross'};
      if ($numcross > 0) {
          my @bgcolors=("#eeeeee","#cccccc");
          $response .= qq(
           You indicated that you wish to add an additional $numcross crosslisting(s).  For each new crosslisting enter the insititutional course section code (e.g., fs03zol101001, for section 001 of zol101 for fs03 semester), and the LON-CAPA section/group ID you wish to assign to students who will be enrolled in your LON-CAPA class as a result of their registration in the crosslisted course section. The LON-CAPA section/group ID can be left blank, if you do not wish to tie a section/group ID to this crosslisting.  The institutional course section code should only contain letters and/or numbers, and must be consistent with the scheme adopted by your Domain Coordinator to map course codes (and section numbers) to your institution's student information system.<br/><br/>
           <form name="enter" method="post">
           <table border="0" cellpadding="2" cellspacing="2" width="100%">
             <tr>
              <td>
               <table border="0" cellspacing="3" cellpadding="3">
                <tr bgcolor="#CCCCFF" align="left">
                 <td><b>Crosslisting</b></td>
                 <td><b>LON-CAPA section/groupID</b></td>
                </tr>
          );
          for (my $i=0; $i<$numcross; $i++) {
              my $colflag = $i%2;
              $response .= qq(
                <tr bgcolor="$bgcolors[$colflag]" align="left">
                 <td><input type="text" size="15" name="newcross_$i" /></td>
                 <td><input type="text" size="10" name="newgroupid_$i" /></td>
                </tr>
              );
          }
          $response .= qq(  </table>
              </td>
             </tr>
             <tr>
              <td align="right">
               <input type="button" name="newcross" value="Go" onClick="process('newcross')" />
              </td>
             </tr>
            </table>
            <input type=\"hidden\" name=\"numcross\" value=\"$numcross\">
            <input type=\"hidden\" name=\"action\" value=\"newcross\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form>
          );
      }
  }
  &print_reply($r,$response,$$tasktitleref{$action});
  return;
}

sub print_crosslistings_response () {
  my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
  my %settings = &Apache::lonnet::get('environment',['internal.crosslistings','internal.coursecode','internal.courseowner'],$dom,$crs);
  my @currxlists = ();
  my @xlists = ();
  my @allxlists = ();
  my @badxlists = ();
  my @badowner = ();
  my $numcross = 0;
  my $xliststr =  $settings{'internal.crosslistings'};
  my $coursecode = $settings{'internal.coursecode'};
  my $owner = $settings{'internal.courseowner'};
  my $response = '';

  if ($xliststr =~ m/,/) {
      @allxlists = split/,/,$xliststr;
  } else {
      unless ($xliststr eq '') { $allxlists[0] = $xliststr; }
  }

  if ( exists($ENV{'form.numcross'}) ) {
      $numcross = $ENV{'form.numcross'};
  }

  if ($numcross > 0) {
      for (my $i=0; $i<$numcross; $i++) {
          my $xl = "newcross_".$i;
          my $gp = "newgroupid_".$i;
          if ( exists($ENV{"form.$xl"}) ) {
              my $coursecheck = &localenroll::validate_courseID($ENV{"form.$xl"});
              if ($coursecheck eq 'ok') {
                  my $addcheck = &localenroll::new_course($ENV{"form.$xl"},$owner);
                  if ($addcheck eq 'ok') {
                     push @xlists,$ENV{"form.$xl"}.":".$ENV{"form.$gp"};
                  } else {
                     push @badowner,$ENV{"form.$xl"}.":".$ENV{"form.$gp"};
                  } 
              } else {
                  push @badxlists, $ENV{"form.$xl"}.":".$ENV{"form.$gp"}.":".$coursecheck;
              }
          }
      }
      push @allxlists, @xlists;
  }

  if (@xlists > 0 ) {
      unless ($xliststr eq '') { $xliststr .= ","; }
      if (@xlists > 1) {
          $xliststr .= join(",",@xlists);
      } else {
          $xliststr .= $xlists[0];
      }
      my %cenv = ('internal.crosslistings' => $xliststr);
      my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
      if ($reply !~ /^ok$/) {
          $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/><br/>";
      } else {
          $response = "The courses listed below are now crosslisted with this LON-CAPA course, and students enrolling in these course sections will be automatically added to the class roster for the course, if you have chosen to enable a nightly automated enrollment update.<br/><ul>\n";
          foreach (@allxlists) {
              my ($xlist,$gp) = split/:/,$_;
              $response .= "<li>$xlist - ID: $gp</li>\n";
          }
          $response .= "</ul><br/><br/>\n";
      }
  } else {
      if ($xliststr =~ m/:/) {
          my @oldxlists = ();
          if ($xliststr =~ m/,/) {
              @oldxlists = split/,/,$xliststr;
          } else {
              $oldxlists[0] = $xliststr;
          }
          $response .= "Although no new crosslistings were added, the courses listed below continue to be crosslisted with your LON-CAPA course.<br/><ul>\n";
          foreach (@oldxlists) {
              my ($xlist,$gp) = split/:/,$_;
              $response .= "<li>$xlist - ID: $gp</li>\n";
          }
          $response .= "</ul><br/><br/>\n";
      }
  }
  if (@badxlists > 0) {
      $response .= "The courses listed below could not be included in the crosslistings for this LON-CAPA course, because they are not valid courses according to your institution's official schedule of classes and sections.<br/><ul>\n";
      foreach (@badxlists) {
          my ($xlist,$gp,$prob) = split/:/,$_;
          $response .= "<li>$xlist - ID: $gp - Error: $prob</li>\n";
      }
      $response .= "</ul><br/><br/>\n";
  }

  if (@badowner > 0) {
      $response .= "The courses listed below could not be included in the crosslistings for this LON-CAPA course, because the owner of this course - $owner - does not have rights to view enrollment in those classes as determined by your instititution's policies on access to official classlists.<br/><ul>\n";
      foreach (@badowner) {
          my ($xlist,$gp) = split/:/,$_;
          $response .= "<li>$xlist - ID: $gp</li>\n";
      }
      $response .= "</ul><br/><br/>\n";
  }

  &print_reply($r,$response,$$tasktitleref{$action});
  return;
}

sub print_sections_menu () {
  my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
  my %settings = &Apache::lonnet::get('environment',['internal.sectionnums','internal.coursecode'],$dom,$crs);
  my @currsections = ();
  my @sections = ();
  my $seccount = 0;
  my $removecount = 0;
  my $addcount = 0;
  my $secstr = '';
  my $response = '';
  my $coursecode = $settings{'internal.coursecode'};
  if ($settings{'internal.sectionnums'} =~ m/,/) {
      @currsections = split/,/,$settings{'internal.sectionnums'};
  } elsif ($settings{'internal.sectionnums'} ne '') {
      $currsections[0] = $settings{'internal.sectionnums'};
  }

  if ( exists($ENV{'form.secshow'}) ) {
     for (my $i=0; $i<$ENV{'form.secshow'}; $i++) {
         my $gp = "loncapasec_".$i;
         my $secnum = "secnum_".$i;
         my $sec = "sec_".$i;
         if ( exists( $ENV{"form.$sec"} ) ) {
            my $secentry;
            if ( exists( $ENV{"form.$secnum"} ) ) { 
                $secentry = $ENV{"form.$secnum"}.':';
            }
            if ( exists( $ENV{"form.$gp"} ) ) {
                $secentry .= $ENV{"form.$gp"};
            }
            push @sections, $secentry;
            $seccount ++;
            unless (grep/^$ENV{"form.$secnum"}:/,@currsections) {
                $addcount ++;
            }
         }
     }
     if (@currsections > 0) {
         for (my $i=0; $i<@currsections; $i++) {
             if ($currsections[$i] =~ m/^(\w+:)/ ) {
                my $oldsec  = $1;
                unless (grep/^$oldsec/,@sections) {
                    $removecount ++;
                }
             }
         }
     }
  } elsif (@currsections > 0) {
      for (my $i=0; $i<@currsections; $i++) {
          my $sec = "sec_".$i;
          my $gp = "secgp_".$i;
          if ( exists($ENV{"form.$sec"}) ) {
              my $secentry = '';
              if ($currsections[$i] =~ m/^(\w+:)/ ) {
                  $secentry = $1;
              }
              if ( exists($ENV{"form.$gp"}) ) {
                  $secentry .= $ENV{"form.$gp"};
              }
              push @sections,$secentry;
              $seccount ++;
          } else {
              $removecount ++;
          }
      }
  }
                                                                                                     
  if ($seccount > 1) {
      $secstr = join(",",@sections);
  } else {
      $secstr = $sections[0];
  }
  my %cenv = ('internal.sectionnums' => $secstr);
  my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
  if ($reply !~ /^ok$/) {
       $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
  } else {
      if ($removecount > 0) {
          $response = "A total of $removecount sections have been removed from the list of sections which contribute to enrollment in LON-CAPA course: $realm ($coursecode).<br/><br/>";
      }
      if ($addcount > 0) {
          $response .= "A total of $addcount sections have been added to the list of
sections which contribute to enrollment in LON-CAPA course: $realm ($coursecode).<br/><br/>";
      }
      if ($seccount > 0) { 
          $response .= "Students enrolling in the $seccount section(s) listed below will be automatically added to the class roster for the course, if you have chosen to enable a nightly automated enrollment update.<br/><ul>\n";
          foreach (@sections) {
              my ($sec,$gp) = split/:/,$_;
              $response .= "<li>$sec  - ID: $gp</li>\n";
          }
          $response .= "</ul><br/>\n";
      }
  }

  if ( exists($ENV{'form.numsec'}) ) {
      my $numsec = $ENV{'form.numsec'};
      if ($numsec > 0) {
          my @bgcolors=("#eeeeee","#cccccc");
          $response .= qq(
           You indicated that you wish to incorporate student enrollment in your LON-CAPA course from an additional $numsec section(s).  For each new section enter the insititutional section code (e.g., 004), and the LON-CAPA section/group ID you wish to assign to students who will be enrolled in your LON-CAPA class as a result of their registration in this particular section. The LON-CAPA section/group ID can be left blank, if you do not wish to designate a section/group ID for this course section.  The institutional section code should only contain letters and/or numbers, and must be consistent with the scheme adopted by your Domain Coordinator to map course section numbers to your institution's student information system.<br/><br/> 
           <form name="enter" method="post">
           <table border="0" cellpadding="2" cellspacing="2" width="100%">
             <tr>
              <td>
               <table border="0" cellspacing="3" cellpadding="3">
                <tr bgcolor="#CCCCFF" align="left">
                 <td><b>Section number</b></td>
                 <td><b>LON-CAPA section/groupID</b></td>
                </tr>
          );
          for (my $i=0; $i<$numsec; $i++) {
              my $colflag = $i%2;
              $response .= qq(
                <tr bgcolor="$bgcolors[$colflag]" align="left">
                 <td><input type="text" size="15" name="newsec_$i" /></td>
                 <td><input type="text" size="10" name="newsecgp_$i" /></td>
                </tr>
              );
          }
          $response .= qq(  </table>
              </td>
             </tr>
             <tr>
              <td align="right">
               <input type="button" name="newsections" value="Go" onClick="process('newsections')" />
              </td>
             </tr>
            </table>
            <input type=\"hidden\" name=\"numsec\" value=\"$numsec\">
            <input type=\"hidden\" name=\"action\" value=\"newsections\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form>
          );
      }
  }
  &print_reply($r,$response,$$tasktitleref{$action});
  return;
}

sub print_sections_response () {
  my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
  my %settings = &Apache::lonnet::get('environment',['internal.sectionnums','internal.coursecode'],$dom,$crs);
  my @currsections = ();
  my @sections = ();
  my @allsections = ();
  my $numsec = 0;
  my $secstr =  $settings{'internal.sectionnums'};
  my $coursecode = $settings{'internal.coursecode'};
  my $response = '';
                                                                                                     
  if ($secstr =~ m/,/) {
      @allsections = split/,/,$secstr;
  } else {
      unless ($secstr eq '') { $allsections[0] = $secstr; }
  }

  if ( exists($ENV{'form.numsec'}) ) {
      $numsec = $ENV{'form.numsec'};
  }
                                                                                                     
  if ($numsec > 0) {
      for (my $i=0; $i<$numsec; $i++) {
          my $sec = "newsec_".$i;
          my $gp = "newsecgp_".$i;
          if ( exists($ENV{"form.$sec"}) ) {
              push @sections,$ENV{"form.$sec"}.":".$ENV{"form.$gp"};
          }
      }
      push @allsections, @sections;
  }
                                                                                                     
  if (@sections > 0 ) {
      unless ($secstr eq '') { $secstr .= ","; } 
      if (@sections > 1) {
          $secstr .= join(",",@sections);
      } else {
          $secstr .= $sections[0];
      }
      my %cenv = ('internal.sectionnums' => $secstr);
      my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
      if ($reply !~ /^ok$/) {
          $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
      } else {
          $response = "Students enrolling in the sections listed below will be automatically added to the class roster for LON-CAPA course $realm ($coursecode), if you have chosen to enable a nightly automated enrollment update.<br/><ul>\n";
      }
  }
                                                                                                     
  foreach (@allsections) {
      my ($sec,$gp) = split/:/,$_;
      $response .= "<li>$sec - ID: $gp</li>\n";
  }
  $response .= "</ul><br/>\n";
                                                                                                     
  &print_reply($r,$response,$$tasktitleref{$action});
  return;
}

sub print_photos_response () {
  my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
  my %settings = &Apache::lonnet::get('environment',['internal.showphotos'],$dom,$crs);
  my $currphotos = $settings{'internal.showphotos'};
  my $showphotos = '';
  if ( exists($ENV{'form.showphotos'}) ) {
      $showphotos=$ENV{'form.showphotos'};
  }
                                                                                            
  my $response = "";
  my %cenv = ('internal.showphotos' => $showphotos);
  my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
  if ($reply !~ /^ok$/) {
      $response = "There was a problem processing your requested change. The student photo import setting for this course has been left unchanged.<br/>";
  } else {
      if ($showphotos) {
          if ($currphotos) {
              $response = "Retrieval of student photos is still <b>enabled</b><br/>";
          } else {
              $response = "Retrieval of student photos in now <b>enabled</b><br/>";
          }
      } else {
          if ($currphotos) {
              $response = "Retrieval of student photos is now <b>disabled</b><br/>";
          } else {
              $response = "Retrieval of student photos is still <b>disabled</b><br/>";
          }
      }
  }
  &print_reply($r,$response,$$tasktitleref{$action});
  return;
}

sub print_update_result () {
  my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
  my $response = '';
  my $updateadds = 0;
  my $updatedrops = 0;
  my $changecount = 0;
  my %affiliates = ();
  my %reply = ();
  my @allcourses = ();
  my %LC_code = ();
  my $logmsg = '';
  my $newusermsg = '';

  my %settings = &Apache::lonnet::get('environment',['internal.coursecode','internal.sectionnums','internal.crosslistings','internal.authtype','internal.autharg','internal.autostart','internal.autoend'],$dom,$crs);
  my $coursecode = $settings{'internal.coursecode'};
  my $authtype = $settings{'internal.authtype'};
  my $autharg = $settings{'internal.autharg'};
  my $autostart = $settings{'internal.autostart'};
  my $autoend = $settings{'internal.autoend'};
 
  if ( exists($ENV{'form.updateadds'}) ) {
      $updateadds = $ENV{'form.updateadds'};
  }
  if ( exists($ENV{'form.updatedrops'}) ) {
      $updatedrops = $ENV{'form.updatedrops'};
  }
  if (!$updateadds && !$updatedrops) {
      $response = "An update of the class roster has not been carried out because you indicated that you wanted to neither add new students, nor expire dropped students based on a comparison between the institutional class lists for the course sections and crosslisted courses that contribure enrollment to this LON-CAPA course.";
  } elsif ($coursecode eq '') {
      $response = "There was a problem retrieving the course code for this LON-CAPA course.  An update of the class roster has not been carried out, and enrollment remains unchanged";
  } else {
# Get complete list of course sections to update
      my @currsections = ();
      my @currxlists = ();

      if ($settings{'internal.sectionnums'} =~ m/,/) {
          @currsections = split/,/,$settings{'internal.sectionnums'};
      } elsif ($settings{'internal.sectionnums'} ne '') {
          $currsections[0] = $settings{'internal.sectionnums'};
      }

      if ($settings{'internal.crosslistings'} =~ m/,/) {
          @currxlists = split/,/,$settings{'internal.crosslistings'};
      } elsif ($settings{'internal.crosslistings'} ne '') {
          $currxlists[0] = $settings{'internal.crosslistings'};
      }
                                                                                                     
      if (@currxlists > 0) {
          foreach (@currxlists) {
              if (m/^(\w+):(\w*)$/) {
                  unless (grep/^$1$/,@allcourses) { 
                      push @allcourses,$1;
                      $LC_code{$1} = $2; 
                  }
              }
          }
      }

      if (@currsections > 0) {
          foreach (@currsections) {
              if (m/^(\w+):(\w*)$/) {
                  my $sec = $coursecode.$1;
                  my $gp = $2;
                  unless (grep/^$sec$/,@allcourses) { 
                      push @allcourses,$sec;
                      $LC_code{$sec} = $gp; 
                  }
              }
          }
      }

      if (@allcourses > 0) {
          @{$affiliates{$crs}} = @allcourses;
          &localenroll::fetch_enrollment($dom,\%affiliates,\%reply);
          if ($reply{$crs} > 0) {
              ($changecount,$response) = &LONCAPA::Enrollment::update_LC($dom,$crs,$updateadds,$updatedrops,$autostart,$autoend,$authtype,$autharg,\@allcourses,\%LC_code,\$logmsg,\$newusermsg,"updatenow");
          } else {
              $response = "There was a problem retrieving institutional class list data for the course sections and crosslisted courses which contribute enrollment to this course. No updates have been carried out, and the roster remains unchanged.";
          }  
      } else {
          $response = "There are currently no course sections or crosslisted courses designated as contributors to enrollment in this LON-CAPA course. As a result a student roster update has not been carried out for $realm ($coursecode)";
      }
  }
  unless ($logmsg eq '') {
      my $loglength = length($logmsg);
      $logmsg = substr($logmsg,0,$loglength-4);
      $logmsg = "<br/><br/>The following messages were generated by the roster update process: <br/><ul><li>".$logmsg."</ul><br/>";
  }
  unless ($newusermsg eq '') {
      $newusermsg = substr( $newusermsg,0,rindex($newusermsg,'<li>') );
      $newusermsg = "<br/><br/>The following new system user(s) who was/were created will be using internal authentication with an initial randomly generated password. A valid e-mail address was not available for this/these user(s) so LON-CAPA account credentials could not be sent via e-mail.<br/><ul><li>".$newusermsg."</ul><br/>";
  }
  $response .= $logmsg.$newusermsg;
  &print_reply($r,$response,$$tasktitleref{$action});
  return;
}

sub print_reply () {
  my ($r,$response,$caller) = @_;
  $r->print("
            <br/><table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
             <tr>
              <tr align=\"left\"><b>$caller</b> - result<br/><br/>$response</td>
              </td>
             </tr>
            </table>
  ");
  return;
}

sub setup_date_selectors {
    my ($starttime,$endtime) = @_;
    if ($endtime == 0) {
        $endtime = 1999915200;
    }
    if (! defined($starttime)) {
        $starttime = time;
        if (exists($ENV{'course.'.$ENV{'request.course.id'}.
                            '.default_enrollment_start_date'})) {
            $starttime = $ENV{'course.'.$ENV{'request.course.id'}.
                                  '.default_enrollment_start_date'};
        }
    }
    if (! defined($endtime)) {
        $endtime = time+(6*30*24*60*60); # 6 months from now, approx
        if (exists($ENV{'course.'.$ENV{'request.course.id'}.
                            '.default_enrollment_end_date'})) {
            $endtime = $ENV{'course.'.$ENV{'request.course.id'}.
                                '.default_enrollment_end_date'};
        }
    }
    my $startdateform = &Apache::lonhtmlcommon::date_setter('enter',
                                                            'startdate',
                                                            $starttime);
    my $enddateform = &Apache::lonhtmlcommon::date_setter('enter',
                                                          'enddate',
                                                          $endtime);
    return ($startdateform,$enddateform);
}

sub get_dates_from_form {
    my $startdate = &Apache::lonhtmlcommon::get_date_from_form('startdate');
    my $enddate   = &Apache::lonhtmlcommon::get_date_from_form('enddate');
    if ( exists ($ENV{'form.no_end_date'}) ) {
        $enddate = 0;
    }
    return ($startdate,$enddate);
}

sub date_setting_table {
    my ($starttime,$endtime) = @_;
    my ($startform,$endform) = &setup_date_selectors($starttime,$endtime);
    my $perpetual = '<nobr><input type="checkbox" name="no_end_date"';
    if (defined($endtime) && $endtime == 0) {
        $perpetual .= ' checked';
    }
    $perpetual.= ' />'.' no ending date</nobr>';
    my $start_table = '';
    $start_table .= "<table>\n";
    $start_table .= '<tr><td align="right">Starting Date</td>'.
        '<td>'.$startform.'</td>'.
        '<td>&nbsp;</td>'."</tr>\n";
    $start_table .= "</table>";
    my $end_table = '';
    $end_table .= "<table>\n";
    $end_table .= '<tr><td align="right">Ending Date</td>'.
        '<td>'.$endform.'</td>'.
        '<td>'.$perpetual.'</td>'."</tr>\n";
    $end_table .= "</table>\n";
    return ($start_table, $end_table);
}

###################################################################
sub handler {
    my $r = shift;
    if ($r->header_only) {
        $r->content_type('text/html');
        $r->send_http_header;
        return OK;
    }
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['action','state']);
    #  Needs to be in a course
    if (! (($ENV{'request.course.fn'}) &&  (&Apache::lonnet::allowed('cst',$ENV{'request.course.id'})))) {
        # Not in a course, or not allowed to modify parms
        $ENV{'user.error.msg'}="/adm/populate:cst:0:0:Cannot drop or add students";
        return HTTP_NOT_ACCEPTABLE; 
    }
    # Start page
    $r->content_type('text/html');
    $r->send_http_header;

    my @tasks = ("information","chgsettings","setdates","notify","crosslist","sections","photos","updatenow");
                                                                                 
    my %tasklong = (
               information => "Task information",
               chgsettings => "Automated adds/drops",
               setdates => "Change enrollment dates",
               notify => "Notification of changes",
               crosslist => "Change crosslistings",
               sections => "Section settings",
               photos => "Student photos",
               updatenow => "Update roster now",
               newcross => "Add crosslistings",
               newsections => "Add sections"
    );
                                                                                 
    my %tasktitle = (
               chgsettings => "Changes to nightly automated enrollments",
               setdates => "Changes to first and/or last automated enrollment dates",
               notify => "Notification of enrollment changes",
               crosslist => "Changes to crosslistings",
               sections => "Changes to section settings",
               photos => "Automatic import of student photos",
               updatenow => "Immediate course roster update",
               newcross => "Adding new crosslisted courses",
               newsections => "Adding new course sections"
    );

    my $realm = '';
    if ( exists($ENV{'request.course.id'}) ) {
        $realm= $ENV{'course.'.$ENV{'request.course.id'}.'.description'};
    }
    unless ($realm) { $realm='&nbsp;'; }
    my $dom = $ENV{'course.'.$ENV{'request.course.id'}.'.domain'};
    my $crs = $ENV{'course.'.$ENV{'request.course.id'}.'.num'};
    
    #
    # Main switch on form.action and form.state, as appropriate
    #

    my $action = "information";
    if ( exists($ENV{'form.action'}) ) {
        $action = $ENV{'form.action'};
    }
    my $state = "choose";

    if ( exists($ENV{'form.state'}) ) {
        $state = $ENV{'form.state'};
    }

    if ($action eq "information") {
        $r->print(&header()); 
    } else {
        if ($state eq "choose") {
            $r->print(&choose_header());
        } else {
            if ($action eq "crosslist") {
                if ( exists($ENV{'form.numcross'}) ) {
                    if ( $ENV{'form.numcross'} > 0 ) {
                        $r->print(&choose_header());
                    } else {
                        $r->print(&header());
                    }
                } else {
                    $r->print(&header());
                }
            } elsif ($action eq "sections") {
                if ( exists($ENV{'form.numsec'}) ) {
                    if ( $ENV{'form.numsec'} > 0 ) {
                        $r->print(&choose_header());
                    } else {
                        $r->print(&header());
                    }
                } else {
                    $r->print(&header());
                }
            } else {
                $r->print(&header());
            }
        }
    }
  
    my $reply = 0;
    unless ($state eq "choose") { $reply = 1; }

    &print_mainbox($r,\%tasklong,$realm,$reply);
    &print_navmenu($r,\@tasks,\%tasklong);
    
    if (($state eq "choose") || ($action eq "information")) {
        &print_main_frame($r,$realm,$dom,$crs,\%tasktitle);
    } elsif ($action eq "chgsettings") {
        &print_chgsettings_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "setdates") {
        &print_setdates_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "notify") {
        &print_notify_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "sections") {
        &print_sections_menu($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "crosslist") {
        &print_crosslistings_menu($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "updatenow") {
        &print_update_result($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "photos") {
        &print_photos_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "newcross") {
        &print_crosslistings_response($r,$realm,$dom,$crs,$action,\%tasktitle);    
    } elsif ($action eq "newsections") {
        &print_sections_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    }
    &print_doc_base($r);  
    return OK;
}
###################################################################
1;
