# The LearningOnline Network
# Printout
#
# $Id: lonprintout.pm,v 1.265 2003/11/11 19:32:22 sakharuk Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
#
package Apache::lonprintout;

use strict;
use Apache::Constants qw(:common :http);
use Apache::lonxml;
use Apache::lonnet;
use Apache::loncommon;
use Apache::inputtags;
use Apache::grades;
use Apache::edit;
use Apache::File();
use Apache::lonnavmaps;
use Apache::lonratedt;
use POSIX qw(strftime);
use Apache::lonlocal;
use GDBM_File;


my %hash;
my $LaTeXwidth = 0;


sub latex_header_footer_remove {
    my $text = shift;
    $text =~ s/\\end{document}//;
    $text =~ s/\\documentclass([^&]*)\\begin{document}//;
    return $text;
}


sub character_chart {
    my $result = shift;	
    $result =~ s/&\#0?0?(7|9);//g;
    $result =~ s/&\#0?(10|13);//g;
    $result =~ s/&\#0?32;/ /g;
    $result =~ s/&\#0?33;/!/g;
    $result =~ s/&(\#0?34|quot);/\"/g;
    $result =~ s/&\#0?35;/\\\#/g;
    $result =~ s/&\#0?36;/\\\$/g;
    $result =~ s/&\#0?37;/\\%/g; 
    $result =~ s/&(\#0?38|amp);/\\&/g; 
    $result =~ s/&\#(0?39|146);/\'/g;
    $result =~ s/&\#0?40;/(/g;
    $result =~ s/&\#0?41;/)/g;
    $result =~ s/&\#0?42;/\*/g;
    $result =~ s/&\#0?43;/\+/g;
    $result =~ s/&\#(0?44|130);/,/g;
    $result =~ s/&\#0?45;/-/g;
    $result =~ s/&\#0?46;/\./g;
    $result =~ s/&\#0?47;/\//g;
    $result =~ s/&\#0?48;/0/g;
    $result =~ s/&\#0?49;/1/g;
    $result =~ s/&\#0?50;/2/g;
    $result =~ s/&\#0?51;/3/g;
    $result =~ s/&\#0?52;/4/g;
    $result =~ s/&\#0?53;/5/g;
    $result =~ s/&\#0?54;/6/g;
    $result =~ s/&\#0?55;/7/g;
    $result =~ s/&\#0?56;/8/g;
    $result =~ s/&\#0?57;/9/g;
    $result =~ s/&\#0?58;/:/g;
    $result =~ s/&\#0?59;/;/g;
    $result =~ s/&(\#0?60|lt|\#139);/\$<\$/g;
    $result =~ s/&\#0?61;/\$=\$/g;
    $result =~ s/&(\#0?62|gt|\#155);/\$>\$/g;
    $result =~ s/&\#0?63;/\?/g;
    $result =~ s/&\#0?65;/A/g;
    $result =~ s/&\#0?66;/B/g;
    $result =~ s/&\#0?67;/C/g;
    $result =~ s/&\#0?68;/D/g;
    $result =~ s/&\#0?69;/E/g;
    $result =~ s/&\#0?70;/F/g;
    $result =~ s/&\#0?71;/G/g;
    $result =~ s/&\#0?72;/H/g;
    $result =~ s/&\#0?73;/I/g;
    $result =~ s/&\#0?74;/J/g;
    $result =~ s/&\#0?75;/K/g;
    $result =~ s/&\#0?76;/L/g;
    $result =~ s/&\#0?77;/M/g;
    $result =~ s/&\#0?78;/N/g;
    $result =~ s/&\#0?79;/O/g;
    $result =~ s/&\#0?80;/P/g;
    $result =~ s/&\#0?81;/Q/g;
    $result =~ s/&\#0?82;/R/g;
    $result =~ s/&\#0?83;/S/g;
    $result =~ s/&\#0?84;/T/g;
    $result =~ s/&\#0?85;/U/g;
    $result =~ s/&\#0?86;/V/g;
    $result =~ s/&\#0?87;/W/g;
    $result =~ s/&\#0?88;/X/g;
    $result =~ s/&\#0?89;/Y/g;
    $result =~ s/&\#0?90;/Z/g;
    $result =~ s/&\#0?91;/[/g;
    $result =~ s/&\#0?92;/\$\\setminus\$/g;
    $result =~ s/&\#0?93;/]/g;
    $result =~ s/&\#(0?94|136);/\$\\wedge\$/g;
    $result =~ s/&\#(0?95|138|154);/\\underline{\\makebox[2mm]{\\strut}}/g;
    $result =~ s/&\#(0?96|145);/\`/g;
    $result =~ s/&\#0?97;/a/g;
    $result =~ s/&\#0?98;/b/g;
    $result =~ s/&\#0?99;/c/g;
    $result =~ s/&\#100;/d/g;
    $result =~ s/&\#101;/e/g;
    $result =~ s/&\#102;/f/g;
    $result =~ s/&\#103;/g/g;
    $result =~ s/&\#104;/h/g;
    $result =~ s/&\#105;/i/g;
    $result =~ s/&\#106;/j/g;
    $result =~ s/&\#107;/k/g;
    $result =~ s/&\#108;/l/g;
    $result =~ s/&\#109;/m/g;
    $result =~ s/&\#110;/n/g;
    $result =~ s/&\#111;/o/g;
    $result =~ s/&\#112;/p/g;
    $result =~ s/&\#113;/q/g;
    $result =~ s/&\#114;/r/g;
    $result =~ s/&\#115;/s/g;
    $result =~ s/&\#116;/t/g;
    $result =~ s/&\#117;/u/g;
    $result =~ s/&\#118;/v/g;
    $result =~ s/&\#119;/w/g;
    $result =~ s/&\#120;/x/g;
    $result =~ s/&\#121;/y/g;
    $result =~ s/&\#122;/z/g;
    $result =~ s/&\#123;/\\{/g;
    $result =~ s/&\#124;/\|/g;
    $result =~ s/&\#125;/\\}/g;
    $result =~ s/&\#126;/\~/g;
    $result =~ s/&\#131;/\\textflorin /g;
    $result =~ s/&\#132;/\"/g;
    $result =~ s/&\#133;/\$\\ldots\$/g;
    $result =~ s/&\#134;/\$\\dagger\$/g;
    $result =~ s/&\#135;/\$\\ddagger\$/g;
    $result =~ s/&\#137;/\\textperthousand /g;
    $result =~ s/&\#140;/{\\OE}/g;
    $result =~ s/&\#147;/\`\`/g;
    $result =~ s/&\#148;/\'\'/g;
    $result =~ s/&\#149;/\$\\bullet\$/g;
    $result =~ s/&\#150;/--/g;
    $result =~ s/&\#151;/---/g;
    $result =~ s/&\#152;/\$\\sim\$/g;
    $result =~ s/&\#153;/\\texttrademark /g;
    $result =~ s/&\#156;/\\oe/g;
    $result =~ s/&\#159;/\\\"Y/g;
    $result =~ s/&(\#160|nbsp);//g;
    $result =~ s/&(\#161|iexcl);/!\`/g;
    $result =~ s/&(\#162|cent);/\\textcent /g;
    $result =~ s/&(\#163|pound);/\\pounds /g; 
    $result =~ s/&(\#164|curren);/\\textcurrency /g;
    $result =~ s/&(\#165|yen);/\\textyen /g;
    $result =~ s/&(\#166|brvbar);/\\textbrokenbar /g;
    $result =~ s/&(\#167|sect);/\\textsection /g;
    $result =~ s/&(\#168|uml);/\\texthighdieresis /g;
    $result =~ s/&(\#169|copy);/\\copyright /g;
    $result =~ s/&(\#170|ordf);/\\textordfeminine /g;
    $result =~ s/&(\#172|not);/\$\\neg\$/g;
    $result =~ s/&(\#173|shy);/ - /g;
    $result =~ s/&(\#174|reg);/\\textregistered /g;
    $result =~ s/&(\#175|macr);/\$^{-}\$/g;
    $result =~ s/&(\#176|deg);/\$^{\\circ}\$/g;
    $result =~ s/&(\#177|plusmn);/\$\\pm\$/g;
    $result =~ s/&(\#178|sup2);/\$^2\$/g;
    $result =~ s/&(\#179|sup3);/\$^3\$/g;
    $result =~ s/&(\#180|acute);/\\textacute /g;
    $result =~ s/&(\#181|micro);/\$\\mu\$/g;
    $result =~ s/&(\#182|para);/\\P/g;
    $result =~ s/&(\#183|middot);/\$\\cdot\$/g;
    $result =~ s/&(\#184|cedil);/\\c{\\strut}/g;
    $result =~ s/&(\#185|sup1);/\$^1\$/g;
    $result =~ s/&(\#186|ordm);/\\textordmasculine /g;
    $result =~ s/&(\#188|frac14);/\\textonequarter /g;
    $result =~ s/&(\#189|frac12);/\\textonehalf /g;
    $result =~ s/&(\#190|frac34);/\\textthreequarters /g;
    $result =~ s/&(\#191|iquest);/?\`/g;   
    $result =~ s/&(\#192|Agrave);/\\\`{A}/g;  
    $result =~ s/&(\#193|Aacute);/\\\'{A}/g; 
    $result =~ s/&(\#194|Acirc);/\\^{A}/g;
    $result =~ s/&(\#195|Atilde);/\\~{A}/g;
    $result =~ s/&(\#196|Auml);/\\\"{A}/g; 
    $result =~ s/&(\#197|Aring);/{\\AA}/g;
    $result =~ s/&(\#198|AElig);/{\\AE}/g;
    $result =~ s/&(\#199|Ccedil);/\\c{c}/g;
    $result =~ s/&(\#200|Egrave);/\\\`{E}/g;  
    $result =~ s/&(\#201|Eacute);/\\\'{E}/g;    
    $result =~ s/&(\#202|Ecirc);/\\^{E}/g;
    $result =~ s/&(\#203|Euml);/\\\"{E}/g;
    $result =~ s/&(\#204|Igrave);/\\\`{I}/g;
    $result =~ s/&(\#205|Iacute);/\\\'{I}/g;    
    $result =~ s/&(\#206|Icirc);/\\^{I}/g;
    $result =~ s/&(\#207|Iuml);/\\\"{I}/g;    
    $result =~ s/&(\#209|Ntilde);/\\~{N}/g;
    $result =~ s/&(\#210|Ograve);/\\\`{O}/g;
    $result =~ s/&(\#211|Oacute);/\\\'{O}/g;
    $result =~ s/&(\#212|Ocirc);/\\^{O}/g;
    $result =~ s/&(\#213|Otilde);/\\~{O}/g;
    $result =~ s/&(\#214|Ouml);/\\\"{O}/g;    
    $result =~ s/&(\#215|times);/\$\\times\$/g;
    $result =~ s/&(\#216|Oslash);/{\\O}/g;
    $result =~ s/&(\#217|Ugrave);/\\\`{U}/g;    
    $result =~ s/&(\#218|Uacute);/\\\'{U}/g;
    $result =~ s/&(\#219|Ucirc);/\\^{U}/g;
    $result =~ s/&(\#220|Uuml);/\\\"{U}/g;
    $result =~ s/&(\#221|Yacute);/\\\'{Y}/g;
    $result =~ s/&(\#223|szlig);/\\ss/g;
    $result =~ s/&(\#224|agrave);/\\\`{a}/g;
    $result =~ s/&(\#225|aacute);/\\\'{a}/g;
    $result =~ s/&(\#226|acirc);/\\^{a}/g;
    $result =~ s/&(\#227|atilde);/\\~{a}/g;
    $result =~ s/&(\#228|auml);/\\\"{a}/g;
    $result =~ s/&(\#229|aring);/{\\aa}/g;
    $result =~ s/&(\#230|aelig);/{\\ae}/g;
    $result =~ s/&(\#231|ccedil);/\\c{c}/g;
    $result =~ s/&(\#232|egrave);/\\\`{e}/g;
    $result =~ s/&(\#233|eacute);/\\\'{e}/g;
    $result =~ s/&(\#234|ecirc);/\\^{e}/g;
    $result =~ s/&(\#235|euml);/\\\"{e}/g;
    $result =~ s/&(\#236|igrave);/\\\`{i}/g;
    $result =~ s/&(\#237|iacute);/\\\'{i}/g;
    $result =~ s/&(\#238|icirc);/\\^{i}/g;
    $result =~ s/&(\#239|iuml);/\\\"{i}/g;
    $result =~ s/&(\#240|eth);/\$\\partial\$/g;
    $result =~ s/&(\#241|ntilde);/\\~{n}/g;
    $result =~ s/&(\#242|ograve);/\\\`{o}/g;
    $result =~ s/&(\#243|oacute);/\\\'{o}/g;
    $result =~ s/&(\#244|ocirc);/\\^{o}/g;
    $result =~ s/&(\#245|otilde);/\\~{o}/g;
    $result =~ s/&(\#246|ouml);/\\\"{o}/g;
    $result =~ s/&(\#247|divide);/\$\\div\$/g;
    $result =~ s/&(\#248|oslash);/{\\o}/g;
    $result =~ s/&(\#249|ugrave);/\\\`{u}/g; 
    $result =~ s/&(\#250|uacute);/\\\'{u}/g;
    $result =~ s/&(\#251|ucirc);/\\^{u}/g;
    $result =~ s/&(\#252|uuml);/\\\"{u}/g;
    $result =~ s/&(\#253|yacute);/\\\'{y}/g;
    $result =~ s/&(\#255|yuml);/\\\"{y}/g;
    $result =~ s/&\#952;/\$\\theta\$/g;
#Greek Alphabet
    $result =~ s/&(alpha|\#945);/\$\\alpha \$/g;
    $result =~ s/&(beta|\#946);/\$\\beta \$/g;
    $result =~ s/&(gamma|\#947);/\$\\gamma \$/g;
    $result =~ s/&(delta|\#948);/\$\\delta \$/g;
    $result =~ s/&(epsilon|\#949);/\$\\epsilon \$/g;
    $result =~ s/&(zeta|\#950);/\$\\zeta \$/g;
    $result =~ s/&(eta|\#951);/\$\\eta \$/g;
    $result =~ s/&(theta|\#952);/\$\\theta \$/g;
    $result =~ s/&(iota|\#953);/\$\\iota \$/g;
    $result =~ s/&(kappa|\#954);/\$\\kappa \$/g;
    $result =~ s/&(lambda|\#955);/\$\\lambda \$/g;
    $result =~ s/&(mu|\#956);/\$\\mu \$/g;
    $result =~ s/&(nu|\#957);/\$\\nu \$/g;
    $result =~ s/&(xi|\#958);/\$\\xi \$/g;
    $result =~ s/&(omicron|\#959);/o/g;
    $result =~ s/&(pi|\#960);/\$\\pi \$/g;
    $result =~ s/&(rho|\#961);/\$\\rho \$/g;
    $result =~ s/&(sigma|\#963);/\$\\sigma \$/g;
    $result =~ s/&(tau|\#964);/\$\\tau \$/g;
    $result =~ s/&(upsilon|\#965);/\$\\upsilon \$/g;
    $result =~ s/&(phi|\#966);/\$\\phi \$/g;
    $result =~ s/&(chi|\#967);/\$\\chi \$/g;
    $result =~ s/&(psi|\#968);/\$\\psi \$/g;
    $result =~ s/&(omega|\#969);/\$\\omega \$/g;
    $result =~ s/&(thetasym|\#977);/\$\\vartheta \$/g;
    $result =~ s/&(piv|\#982);/\$\\varpi \$/g;
    $result =~ s/&(Alpha|\#913);/A/g;
    $result =~ s/&(Beta|\#914);/B/g;
    $result =~ s/&(Gamma|\#915);/\$\\Gamma \$/g;
    $result =~ s/&(Delta|\#916);/\$\\Delta \$/g;
    $result =~ s/&(Epsilon|\#917);/E/g;
    $result =~ s/&(Zeta|\#918);/Z/g;
    $result =~ s/&(Eta|\#919);/H/g;
    $result =~ s/&(Theta|\#920);/\$\\Theta \$/g;
    $result =~ s/&(Iota|\#921);/I/g;
    $result =~ s/&(Kappa|\#922);/K/g;
    $result =~ s/&(Lambda|\#923);/\$\\Lambda \$/g;
    $result =~ s/&(Mu|\#924);/M/g;
    $result =~ s/&(Nu|\#925);/N/g;
    $result =~ s/&(Xi|\#926);/\$\\Xi \$/g;
    $result =~ s/&(Omicron|\#927);/O/g;
    $result =~ s/&(Pi|\#928);/\$\\Pi \$/g;
    $result =~ s/&(Rho|\#929);/P/g;
    $result =~ s/&(Sigma|\#931);/\$\\Sigma \$/g;
    $result =~ s/&(Tau|\#932);/T/g;
    $result =~ s/&(Upsilon|\#933);/\$\\Upsilon \$/g;
    $result =~ s/&(Phi|\#934);/\$\\Phi \$/g;
    $result =~ s/&(Chi|\#935);/X/g;
    $result =~ s/&(Psi|\#936);/\$\\Psi \$/g;
    $result =~ s/&(Omega|\#937);/\$\\Omega \$/g;
#Arrows (extended HTML 4.01)
    $result =~ s/&(larr|\#8592);/\$\\leftarrow \$/g;
    $result =~ s/&(uarr|\#8593);/\$\\uparrow \$/g;
    $result =~ s/&(rarr|\#8594);/\$\\rightarrow \$/g;
    $result =~ s/&(darr|\#8595);/\$\\downarrow \$/g;
    $result =~ s/&(harr|\#8596);/\$\\leftrightarrow \$/g;
    $result =~ s/&(lArr|\#8656);/\$\\Leftarrow \$/g;
    $result =~ s/&(uArr|\#8657);/\$\\Uparrow \$/g;
    $result =~ s/&(rArr|\#8658);/\$\\Rightarrow \$/g;
    $result =~ s/&(dArr|\#8659);/\$\\Downarrow \$/g;
    $result =~ s/&(hArr|\#8660);/\$\\Leftrightarrow \$/g;
#Mathematical Operators (extended HTML 4.01)
    $result =~ s/&(forall|\#8704);/\$\\forall \$/g;
    $result =~ s/&(part|\#8706);/\$\\partial \$/g;
    $result =~ s/&(exist|\#8707);/\$\\exists \$/g;
    $result =~ s/&(empty|\#8709);/\$\\emptyset \$/g;
    $result =~ s/&(nabla|\#8711);/\$\\nabla \$/g;
    $result =~ s/&(isin|\#8712);/\$\\in \$/g;
    $result =~ s/&(notin|\#8713);/\$\\notin \$/g;
    $result =~ s/&(ni|\#8715);/\$\\ni \$/g;
    $result =~ s/&(prod|\#8719);/\$\\prod \$/g;
    $result =~ s/&(sum|\#8721);/\$\\sum \$/g;
    $result =~ s/&(minus|\#8722);/\$-\$/g;
    $result =~ s/&(lowast|\#8727);/\$*\$/g;
    $result =~ s/&(radic|\#8730);/\$\\surd \$/g;
    $result =~ s/&(prop|\#8733);/\$\\propto \$/g;
    $result =~ s/&(infin|\#8734);/\$\\infty \$/g;
    $result =~ s/&(ang|\#8736);/\$\\angle \$/g;
    $result =~ s/&(and|\#8743);/\$\\wedge \$/g;
    $result =~ s/&(or|\#8744);/\$\\vee \$/g;
    $result =~ s/&(cap|\#8745);/\$\\cap \$/g;
    $result =~ s/&(cup|\#8746);/\$\\cup \$/g;
    $result =~ s/&(int|\#8747);/\$\\int \$/g;
    $result =~ s/&(sim|\#8764);/\$\\sim \$/g;
    $result =~ s/&(cong|\#8773);/\$\\cong \$/g;
    $result =~ s/&(asymp|\#8776);/\$\\approx \$/g;
    $result =~ s/&(ne|\#8800);/\$\\not= \$/g;
    $result =~ s/&(equiv|\#8801);/\$\\equiv \$/g;
    $result =~ s/&(le|\#8804);/\$\\leq \$/g;
    $result =~ s/&(ge|\#8805);/\$\\geq \$/g;
    $result =~ s/&(sub|\#8834);/\$\\subset \$/g;
    $result =~ s/&(sup|\#8835);/\$\\supset \$/g;
    $result =~ s/&(nsub|\#8836);/\$\\not\\subset \$/g;
    $result =~ s/&(sube|\#8838);/\$\\subseteq \$/g;
    $result =~ s/&(supe|\#8839);/\$\\supseteq \$/g;
    $result =~ s/&(oplus|\#8853);/\$\\oplus \$/g;
    $result =~ s/&(otimes|\#8855);/\$\\otimes \$/g;
    $result =~ s/&(perp|\#8869);/\$\\perp \$/g;
    $result =~ s/&(sdot|\#8901);/\$\\cdot \$/g;
#Geometric Shapes (extended HTML 4.01)
    $result =~ s/&(loz|\#9674);/\$\\Diamond \$/g;
#Miscellaneous Symbols (extended HTML 4.01)
    $result =~ s/&(spades|\#9824);/\$\\spadesuit \$/g;
    $result =~ s/&(clubs|\#9827);/\$\\clubsuit \$/g;
    $result =~ s/&(hearts|\#9829);/\$\\heartsuit \$/g;
    $result =~ s/&(diams|\#9830);/\$\\diamondsuit \$/g;
    return $result;
}


sub page_format {
#
#Correspondence between $papersize variable and real paper format:
# 0 - "Letter [8 1/2x11 in]"
# 1 - "Legal [8 1/2x14 in]"
# 2 - "Ledger/Tabloid [11x17 in]"
# 3 - "Executive [7 1/2x10 in]"
# 4 - "A2 [420x594 mm]"
# 5 - "A3 [297x420 mm]"
# 6 - "A4 [210x297 mm]"
# 7 - "A5 [148x210 mm]"
# 8 - "A6 [105x148 mm]"
# 
    my ($papersize,$layout,$numberofcolumns) = @_; 
    my ($textwidth,$textheight,$oddoffset,$evenoffset,$topmargin) = (0,0,0,0,0);
    if ($papersize eq '0') {
	if ($layout eq 'book') {
	    if ($numberofcolumns == 1) {
		$textwidth = '7.1 in';      #'18 cm';
		$textheight = '10.2 in';    #'25.9 cm';
		$oddoffset = '-0.57 in';
		$evenoffset = '-0.57 in';
	    } elsif ($numberofcolumns == 2) {
		$textwidth = '3.66 in';     #'93 mm';
		$textheight = '10.2 in';    #'25.9 cm';
		$oddoffset = '-0.57 in';
		$evenoffset = '-0.57 in';
	    }
	} elsif ($layout eq 'album') {
	    if ($numberofcolumns eq '1') {
		$textwidth = '8.8 in';
		$textheight = '6.8 in';
		$oddoffset = '-40 pt';
		$evenoffset = '-60 pt';
	    } elsif ($numberofcolumns == 2) {
		$textwidth = '4.4 in';
		$textheight = '6.8 in';
		$oddoffset = '-0.5 in';
		$evenoffset = '-1.5 in';
		$topmargin = '3.5 in';
	    }
	}
#    } elsif($papersize eq '1') {
#    } elsif($papersize eq '2') {
#    } elsif($papersize eq '3'/) {
    } elsif($papersize eq '6') {
	if ($layout eq 'book') {
	    if ($numberofcolumns == 1) {
		$textwidth = '18 cm';
		$textheight = '28 cm';
		$oddoffset = '-0.57 in';
		$evenoffset = '-0.57 in';
	    } elsif ($numberofcolumns == 2) {
		$textwidth = '96 mm';
		$textheight = '2 cm';
		$oddoffset = '-0.57 in';
		$evenoffset = '-0.57 in';
	    }
	} elsif ($layout eq 'album') {
	    if ($numberofcolumns eq '1') {
		$textwidth = '8.5 in';
		$textheight = '7.7 in';
		$oddoffset = '-40 pt';
		$evenoffset = '-60 pt';
	    } elsif ($numberofcolumns == 2) {
		$textwidth = '3.9 in';
		$textheight = '7.7 in';
		$oddoffset = '-40 pt';
		$evenoffset = '-60 pt';
	    }
	}
#    } elsif($papersize eq '5') {
#    } elsif($papersize eq '4') {
#    } elsif($papersize eq '7') {
#    } elsif($papersize eq '8') {
    }
    return $textwidth,$textheight,$oddoffset,$evenoffset,$topmargin;
}


sub get_name {
    my ($uname,$udom)=@_;
    if (!defined($uname)) { $uname=$ENV{'user.name'}; }
    if (!defined($udom)) { $udom=$ENV{'user.domain'}; }
    my $plainname=&Apache::loncommon::plainname($uname,$udom);
    if ($plainname=~/^\s*$/) { $plainname=$uname.'@'.$udom; }
    $plainname=&Apache::lonxml::latex_special_symbols($plainname,'header');
    return $plainname;
}

sub get_course {
    my $courseidinfo;
    if (defined($ENV{'request.course.id'})) {
	$courseidinfo = &Apache::lonxml::latex_special_symbols(&Apache::lonnet::unescape($ENV{'course.'.$ENV{'request.course.id'}.'.description'}),'header');
    }
    return $courseidinfo;
}

sub page_format_transformation {
    my ($papersize,$layout,$numberofcolumns,$choice,$text,$assignment,$tableofcontents,$indexlist) = @_; 
    my ($textwidth,$textheight,$oddoffset,$evenoffset,$topmargin);
    $assignment=&Apache::lonxml::latex_special_symbols($assignment,'header');
    ($textwidth,$textheight,$oddoffset,$evenoffset,$topmargin) = &page_format($papersize,$layout,$numberofcolumns,$topmargin);
    my $name = &get_name();
    my $courseidinfo = &get_course();
    if (defined($courseidinfo)) { $courseidinfo=' - '.$courseidinfo }
    if ($layout eq 'album') {
	my $topmargintoinsert = '';
	if ($topmargin ne '0') {$topmargintoinsert='\setlength{\topmargin}{'.$topmargin.'}';}
	    $text =~ s/\\begin{document}/\\setlength{\\oddsidemargin}{$oddoffset}\\setlength{\\evensidemargin}{$evenoffset}$topmargintoinsert\\setlength{\\textwidth}{$textwidth}\\setlength{\\textheight}{$textheight}\\setlength{\\textfloatsep}{8pt plus 2\.0pt minus 4\.0pt}\\newlength{\\minipagewidth}\\setlength{\\minipagewidth}{\\textwidth\/\$number_of_columns-0\.2cm}\\usepackage{fancyhdr}\\pagestyle{fancy}\\rhead{}\\chead{}\\lhead{\\textbf{$name} $courseidinfo \\hfill \\thepage \\\\ \\textit{$assignment}} \\begin{document}\\voffset=-0\.8 cm\\setcounter{page}{1} /;
    } elsif ($layout eq 'book') {
	if ($choice ne 'All class print') { 
	    $text =~ s/\\begin{document}/\\textheight $textheight\\oddsidemargin = $evenoffset\\evensidemargin = $evenoffset\\textwidth= $textwidth\\newlength{\\minipagewidth}\\setlength{\\minipagewidth}{\\textwidth\/\$number_of_columns-0\.2cm}\\renewcommand{\\ref}{\\keephidden\}\\usepackage{fancyhdr}\\pagestyle{fancy}\\rhead{}\\chead{}\\lhead{\\textbf{$name} $courseidinfo \\hfill \\thepage \\\\ \\textit{$assignment}}\\begin{document}\\voffset=-0\.8 cm\\setcounter{page}{1}/;
	} else {
	    $text =~ s/\\pagestyle{fancy}\\rhead{}\\chead{}\s*\\begin{document}/\\textheight = $textheight\\oddsidemargin = $evenoffset\\evensidemargin = $evenoffset\\textwidth= $textwidth\\newlength{\\minipagewidth}\\setlength{\\minipagewidth}{\\textwidth\/\$number_of_columns-0\.2cm}\\renewcommand{\\ref}{\\keephidden\}\\pagestyle{fancy}\\rhead{}\\chead{}\\begin{document}\\voffset=-0\.8cm\\setcounter{page}{1}  \\vskip 5 mm /;
	}
    }
    if ($tableofcontents eq 'yes') {$text=~s/(\\setcounter\{page\}\{1\})/$1 \\tableofcontents\\newpage /;}
    if ($indexlist eq 'yes') {
	$text=~s/(\\begin{document})/\\makeindex $1/;
	$text=~s/(\\end{document})/\\strut\\\\\\strut\\printindex $1/;
    }
    return $text;
}


sub page_cleanup {
    my $result = shift;	
 
    $result =~ m/\\end{document}(\d*)$/;
    my $number_of_columns = $1;
    my $insert = '{';
    for (my $id=1;$id<=$number_of_columns;$id++) { $insert .='l'; }
    $insert .= '}';
    $result =~ s/(\\begin{longtable})INSERTTHEHEADOFLONGTABLE\\endfirsthead\\endhead/$1$insert/g;
    $result =~ s/&\s*REMOVETHEHEADOFLONGTABLE\\\\/\\\\/g;
    return $result,$number_of_columns;
}


sub details_for_menu {

    my $name_of_resourse = $hash{'title_'.$hash{'ids_'.$ENV{'form.postdata'}}};
    my $symbolic = &Apache::lonnet::symbread($ENV{'form.postdata'});
    my ($map,$id,$resource)=&Apache::lonnet::decode_symb($symbolic);
    $map=&Apache::lonnet::clutter($map);
    my $name_of_sequence;
    $name_of_sequence = $hash{'title_'.$hash{'ids_'.$map}};
    if ($name_of_sequence =~ /^\s*$/) {
	$map =~ m|([^/]+)$|;
	$name_of_sequence = $1;
    }
    my $name_of_map = $hash{'title_'.$hash{'ids_'.&Apache::lonnet::clutter($ENV{'request.course.uri'})}};
    if ($name_of_map =~ /^\s*$/) {
	$ENV{'request.course.uri'} =~ m|([^/]+)$|;
	$name_of_map = $1;
    }
    return ($name_of_resourse,$name_of_sequence,$name_of_map);

}


sub latex_corrections {

    my ($number_of_columns,$result) = @_;

#    $result =~ s/\\includegraphics{/\\includegraphics\[width=\\minipagewidth\]{/g;
    $result =~ s/\$number_of_columns/$number_of_columns/g;
    $result =~ s/(\\end{document})/\\strut\\newline\\noindent\\makebox\[\\textwidth\/$number_of_columns\]\[b\]{\\hrulefill}\\newline\\noindent\\tiny Printed from LON-CAPA\\copyright MSU{\\hfill} Licensed under GNU General Public License  $1/;
    $result =~ s/(\\end{longtable}\s*)(\\strut\\newline\\noindent\\makebox\[\\textwidth\/$number_of_columns\]\[b\]{\\hrulefill})/$2$1/g;
    $result =~ s/(\\end{longtable}\s*)\\strut\\newline/$1/g;
#-- LaTeX corrections     
    my $first_comment = index($result,'<!--',0);
    while ($first_comment != -1) {
	my $end_comment = index($result,'-->',$first_comment);
	substr($result,$first_comment,$end_comment-$first_comment+3) = '';
	$first_comment = index($result,'<!--',$first_comment);
    }
    $result =~ s/^\s+$//gm; #remove empty lines
    $result =~ s/(\s)(\s+)/$1/g; #removes more than one empty space
    $result =~ s/\\\\\s*\\vskip/\\vskip/gm;
    $result =~ s/\\\\\s*\\noindent\s*(\\\\)+/\\\\\\noindent /g;
    $result =~ s/{\\par }\s*\\\\/\\\\/gm;
	$result =~ s/\\\\\s+\[/ \[/g;
    $result =~ s/\b__+\b/\\makebox\[1 cm\]\[b\]{\\hrulefill}/g;
    #conversion of html characters to LaTeX equivalents
    if ($result =~ m/&(\w+|#\d+);/) {
	$result = &character_chart($result);
    }
    $result =~ s/(\\end{tabular})\s*\\vskip 0 mm/$1/g;
    $result =~ s/(\\begin{enumerate})\s*\\noindent/$1/g;

    return $result;
}


sub index_table {
    my $currentURL = shift;
    my $insex_string='';
    $currentURL=~s/\.([^\/+])$/\.$1\.meta/;
    $insex_string=&Apache::lonnet::metadata($currentURL,'keywords');
    return $insex_string;
}


sub IndexCreation {
    my ($texversion,$currentURL)=@_;
    my @key_words=split(/,/,&index_table($currentURL));
    my $chunk='';
    my $st=index $texversion,'\addcontentsline{toc}{subsection}{';
    if ($st>0) {
	for (my $i=0;$i<3;$i++) {$st=(index $texversion,'}',$st+1);}
	$chunk=substr($texversion,0,$st+1);
	substr($texversion,0,$st+1)=' ';
    }
    foreach my $key_word (@key_words) {
	if ($key_word=~/\S+/) {
	    $texversion=~s/\b($key_word)\b/$1 \\index{$key_word} /i;
	}
    }			
    if ($st>0) {substr($texversion,0,1)=$chunk;}
    return $texversion;
}

sub print_latex_header {
    my $mode=shift;
    my $output='\documentclass[letterpaper]{article}';
    if ($mode eq 'batchmode') {
	$output.='\batchmode';
    }
    $output.='\newcommand{\keephidden}[1]{}\renewcommand{\deg}{$^{\circ}$}'.
	     '\usepackage{longtable}\usepackage{textcomp}\usepackage{makeidx}'.
	     '\usepackage[dvips]{graphicx}\usepackage{epsfig}\usepackage{calc}'.
	     '\newenvironment{choicelist}{\begin{list}{}{\setlength{\rightmargin}{0in}'.
	     '\setlength{\leftmargin}{0.13in}\setlength{\topsep}{0.05in}'.
	     '\setlength{\itemsep}{0.022in}\setlength{\parsep}{0in}'.
	     '\setlength{\belowdisplayskip}{0.04in}\setlength{\abovedisplayskip}{0.05in}'.
	     '\setlength{\abovedisplayshortskip}{-0.04in}'.
	     '\setlength{\belowdisplayshortskip}{0.04in}}}{\end{list}}'.
	     '\renewenvironment{theindex}{\begin{list}{}{{\vskip 1mm \noindent \large'.
	     '\textbf{Index}} \newline \setlength{\rightmargin}{0in}'.
	     '\setlength{\leftmargin}{0.13in}\setlength{\topsep}{0.01in}'.
	     '\setlength{\itemsep}{0.1in}\setlength{\parsep}{-0.02in}'.
	     '\setlength{\belowdisplayskip}{0.01in}\setlength{\abovedisplayskip}{0.01in}'.
	     '\setlength{\abovedisplayshortskip}{-0.04in}'.
	     '\setlength{\belowdisplayshortskip}{0.01in}}}{\end{list}}\begin{document}';
    return $output;	     
}

sub path_to_problem {
    my ($urlp,$LaTeXwidth)=@_;
    my $newurlp = '';
    $LaTeXwidth=~s/\s*mm\s*$//;
    my $HowMany = length($urlp)*2;
    if ($HowMany > $LaTeXwidth) {
	my @temporrary = split '/',$urlp;
	my $HowManyNew = 0;
	for (my $ii=0;$ii<=$#temporrary;$ii++) {
	    if ($temporrary[$ii] ne '') {
		$HowManyNew += length($temporrary[$ii])*2;
		if ($HowManyNew < $LaTeXwidth ) {
		    $newurlp .=  '/'.$temporrary[$ii];
		} else {
		    $HowManyNew = 0;
		    $newurlp .=  '|\vskip -1 mm \noindent \verb|';
		    $ii--;
		}
	    }
	}
    } else {
	$newurlp=$urlp;
    }
    return '{\small\noindent\verb|'.$newurlp.'|\vskip 0 mm}';
}

sub output_data {
    my ($r,$helper,$rparmhash) = @_;
    my %parmhash = %$rparmhash;
    my $bodytag=&Apache::loncommon::bodytag('Preparing Printout');
    $r->print(<<ENDPART);
<html>
<script type="text/javascript" language="Javascript" >
    var editbrowser;
    function openbrowser(formname,elementname,only,omit) {
        var url = '/res/?';
        if (editbrowser == null) {
            url += 'launch=1&';
        }
        url += 'catalogmode=interactive&';
        url += 'mode=parmset&';
        url += 'form=' + formname + '&';
        if (only != null) {
            url += 'only=' + only + '&';
        } 
        if (omit != null) {
            url += 'omit=' + omit + '&';
        }
        url += 'element=' + elementname + '';
        var title = 'Browser';
        var options = 'scrollbars=1,resizable=1,menubar=0';
        options += ',width=700,height=600';
        editbrowser = open(url,title,options,'1');
        editbrowser.focus();
    }
</script>
<head>
<title>LON-CAPA output for printing</title>
</head>
$bodytag
Please stand by while processing your print request, this may take some time ...
ENDPART

    my $format_from_helper = $helper->{'VARS'}->{'FORMAT'};
    my ($result,$selectionmade) = ('','');
    my $number_of_columns = 1; #used only for pages to determine the width of the cell
    my @temporary_array=split /\|/,$format_from_helper;
    my ($laystyle,$numberofcolumns,$papersize)=@temporary_array;
    if ($laystyle eq 'L') {
	$laystyle='album';
    } else {
	$laystyle='book';
    }
    my ($textwidth,$textheight,$oddoffset,$evenoffset) = &page_format($papersize,$laystyle,$numberofcolumns);
    my $assignment =  $ENV{'form.assignment'};
    my $LaTeXwidth; 
    if ($textwidth=~/(\d+\.?\d*)\s*cm/) {
	$LaTeXwidth = $1*10;
    } elsif ($textwidth=~/(\d+\.?\d*)\s*mm/) {
	$LaTeXwidth = $1;
    } elsif ($textwidth=~/(\d+\.?\d*)\s*in/) {
	$LaTeXwidth = $1*25.4;
    }
    $LaTeXwidth.=' mm';
    
    if ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'current_document') {
      #-- single document - problem, page, html, xml, ...
	my $currentURL;
	if ($helper->{'VARS'}->{'construction'} ne '1') {
            #prints published resource
	    $currentURL=$helper->{'VARS'}->{'postdata'};
	} else {
            #prints resource from the construction space
	    $currentURL='/'.$helper->{'VARS'}->{'filename'};
	    if ($currentURL=~/([^?]+)/) {$currentURL=$1;}
	}
	$selectionmade = 1;
	if ($currentURL=~/\.(problem|exam|quiz|assess|survey|form|library|page|xml|html|htm|xhtml|xhtm)$/) {
	    my $rndseed=time;
	    my $texversion='';
	    if ($helper->{'VARS'}->{'ANSWER_TYPE'} ne 'only') {
		my %moreenv;
		$moreenv{'request.filename'}=$currentURL;
		if ($helper->{'VARS'}->{'style_file'}=~/\w/) {
		    $moreenv{'construct.style'}=$helper->{'VARS'}->{'style_file'};
		    my $dom = $ENV{'user.domain'};
		    my $user = $ENV{'user.name'};
		    my $put_result = &Apache::lonnet::put('environment',{'construct.style'=>$helper->{'VARS'}->{'style_file'}},$dom,$user);
		}
		my %form;
		$form{'grade_target'}='tex';
		$form{'textwidth'}=$LaTeXwidth;	
		$form{'problem_split'}=$parmhash{'problem_stream_switch'};
		$form{'latex_type'}=$helper->{'VARS'}->{'LATEX_TYPE'};
		if ($helper->{'VARS'}->{'curseed'}) {
		    $rndseed=$helper->{'VARS'}->{'curseed'};
		}
		$form{'rndseed'}=$rndseed;
		&Apache::lonnet::appenv(%moreenv);
		&Apache::lonnet::delenv('form.counter');
		&Apache::lonxml::init_counter();
		$texversion=&Apache::lonnet::ssi($currentURL,%form);
		&Apache::lonnet::delenv('form.counter');
		&Apache::lonnet::delenv('request.filename');
	    }
	    if(($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'no') ||
	       ($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'only')) {
		my %form;
		$form{'grade_target'}='answer';
		$form{'answer_output_mode'}='tex';
		$form{'rndseed'}=$rndseed;
		my $answer=&Apache::lonnet::ssi($currentURL,%form);
		if ($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'no') {
		    $texversion=~s/(\\keephidden{ENDOFPROBLEM})/$answer$1/;
		} else {
		    $texversion=&print_latex_header($helper->{'VARS'}->{'LATEX_TYPE'});
		    if ($helper->{'VARS'}->{'construction'} ne '1') {
			$texversion.='\vskip 0 mm \noindent\textbf{'.&Apache::lonnet::gettitle($helper->{'VARS'}->{'symb'}).'}\vskip 0 mm ';
			$texversion.=&path_to_problem ($currentURL,$LaTeXwidth);
		    } else {
			$texversion.='\vskip 0 mm \noindent\textbf{Prints from construction space - there is no title.}\vskip 0 mm ';
			my $URLpath=$currentURL;
			$URLpath=~s/~([^\/]+)/public_html\/$1\/$1/;
			$texversion.=&path_to_problem ($URLpath,$LaTeXwidth);
		    }
		    $texversion.='\vskip 1 mm '.$answer.'\end{document}';
		}
	    }
	    if ($helper->{'VARS'}->{'TABLE_INDEX'} eq 'yes') {
		$texversion=&IndexCreation($texversion,$currentURL);
	    }
	    if ($helper->{'VARS'}->{'CONSTR_RESOURSE_URL'} eq 'yes') {
		$texversion=~s/(\\addcontentsline\{toc\}\{subsection\}\{[^\}]*\})/$1 URL: \\verb|$currentURL| \\strut\\\\\\strut /;

	    }
	    $result .= $texversion;
	    if ($currentURL=~m/\.page\s*$/) {
		($result,$number_of_columns) = &page_cleanup($result);
	    }
        } elsif ($currentURL=~/\.sequence$/ && $helper->{'VARS'}->{'construction'} eq '1') {
            #printing content of sequence from the construction space	
	    my $flag_latex_header_remove = 'NO'; 
	    my $rndseed=time;
	    if ($helper->{'VARS'}->{'curseed'}) {
		$rndseed=$helper->{'VARS'}->{'curseed'};
	    }
	    $currentURL=~s|\/~([^\/]+)\/|\/home\/$1\/public_html\/|;
	    my $errtext=&Apache::lonratedt::mapread($currentURL);
	    for (my $member=0;$member<=$#Apache::lonratedt::order;$member++) {
		$Apache::lonratedt::resources[$Apache::lonratedt::order[$member]]=~/^([^:]*):([^:]*):/;
		my $urlp=$2;
		if ($urlp=~/\.(problem|exam|quiz|assess|survey|form|library|xml|html|htm|xhtml|xhtm)$/) {
		    my $texversion='';
		    if ($helper->{'VARS'}->{'ANSWER_TYPE'} ne 'only') {
			my %form;
			$form{'grade_target'}='tex';
			$form{'textwidth'}=$LaTeXwidth;
			$form{'problem_split'}=$parmhash{'problem_stream_switch'};
			$form{'latex_type'}=$helper->{'VARS'}->{'LATEX_TYPE'};
			$form{'rndseed'}=$rndseed;
			$texversion=&Apache::lonnet::ssi($urlp,%form);
		    }
		    if((($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'no') ||
		       ($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'only')) && 
		       ($urlp=~/\.(problem|exam|quiz|assess|survey|form|library|page)$/)) {
			my %form;
			$form{'grade_target'}='answer';
			$form{'answer_output_mode'}='tex';
			$form{'rndseed'}=$rndseed;
                        if ($urlp=~/\/res\//) {$ENV{'request.state'}='published';}
			my $answer=&Apache::lonnet::ssi($urlp,%form);
			if ($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'no') {
			    $texversion=~s/(\\keephidden{ENDOFPROBLEM})/$answer$1/;
			} else {
			    $texversion=&print_latex_header($helper->{'VARS'}->{'LATEX_TYPE'});
			    $texversion.='\vskip 0 mm \noindent\textbf{'.&Apache::lonnet::gettitle($helper->{'VARS'}->{'symb'}).'}\vskip 0 mm ';
			    $texversion.=&path_to_problem ($currentURL,$LaTeXwidth);
			    $texversion.='\vskip 1 mm '.$answer.'\end{document}';
			}
		    }
		    if ($flag_latex_header_remove ne 'NO') {
			$texversion = &latex_header_footer_remove($texversion);
		    } else {
			$texversion =~ s/\\end{document}//;
		    }
		    if ($helper->{'VARS'}->{'TABLE_INDEX'} eq 'yes') {
			$texversion=&IndexCreation($texversion,$urlp);
		    }
		    if ($helper->{'VARS'}->{'CONSTR_RESOURSE_URpL'} eq 'yes') {
			$texversion=~s/(\\addcontentsline\{toc\}\{subsection\}\{[^\}]*\})/$1 URL: \\verb|$urlp| \\strut\\\\\\strut /;
		    }
		    $result.=$texversion;
		    $flag_latex_header_remove = 'YES';
		} elsif ($urlp=~/\.(sequence|page)$/) {
		    $result.='\strut\newline\noindent Sequence/page '.$urlp.'\strut\newline\noindent\makebox[\textwidth/$number_of_columns][b]{\hrulefill}\newline\noindent ';
		}  
	    }
	    if ($helper->{VARS}->{'construction'} eq '1') {$result=~s/(\\begin{document})/$1 \\fbox\{RANDOM SEED IS $rndseed\} /;}
	    $result .= '\end{document}';   
	} elsif ($currentURL=~/\/smppg$/) { 
		my %form;
		$form{'grade_target'}='tex';
		$form{'textwidth'}=$LaTeXwidth;	
		$form{'latex_type'}=$helper->{'VARS'}->{'LATEX_TYPE'};
		my $texversion=&Apache::lonnet::ssi($currentURL,%form);
		$result .= $texversion;
	} else {
	  $result.='\documentclass[letterpaper]{article}\usepackage{calc}\begin{document}Printout of this type of document is currently not supported: ';
	  if ($currentURL=~/\/(aboutme|syllabus|bulletinboard|smppg)$/) {
	      $result.=$1;
	  } elsif ($currentURL=~/\/ext\//) {
	      $result.=' \strut \\\\ THIS IS EXTERNAL RESOURCE WITH URL \strut \\\\ '.$currentURL.' ';
	  } else {
	      $result.=$currentURL;
	  }
	  $result.=' \end{document}'
	}
    } elsif (($helper->{'VARS'}->{'PRINT_TYPE'} eq 'map_problems') or
             ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'map_problems_pages') or
             ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'all_problems') or
	     ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'select_sequences')) { 
        #-- produce an output string
	my %form=();	   
	$form{'grade_target'}='tex';
	$form{'textwidth'}=$LaTeXwidth;
	$form{'problem_split'}=$parmhash{'problem_stream_switch'};
	$form{'latex_type'}=$helper->{'VARS'}->{'LATEX_TYPE'};
	my $flag_latex_header_remove = 'NO';
	my $flag_page_in_sequence = 'NO';
	my @master_seq=split /\|\|\|/, $helper->{'VARS'}->{'RESOURCES'};
	my $prevassignment='';
	&Apache::lonxml::init_counter();
	for (my $i=0;$i<=$#master_seq;$i++) {
	    my (undef,undef,$urlp)=&Apache::lonnet::decode_symb($master_seq[$i]);
	    $urlp=&Apache::lonnet::clutter($urlp);
	    if ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'map_problems') {
		$selectionmade = 2;
	    } elsif ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'map_problems_pages') {
		$selectionmade = 3;
	    } elsif ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'all_problems') {
		$selectionmade = 4;
	    } elsif ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'select_sequences') {
		$selectionmade = 7;
	    }
	    $form{'symb'}=$master_seq[$i];
	    my ($sequence)=&Apache::lonnet::decode_symb($master_seq[$i]);
	    my $assignment=&Apache::lonxml::latex_special_symbols(&Apache::lonnet::gettitle($sequence),'header'); #tittle of the assignment which contains this problem
	    if ($i==0) {$prevassignment=$assignment;}
	    #&Apache::lonnet::logthis("Trying to get $urlp with symb $master_seq[$i]");
	    my $texversion.=&Apache::lonnet::ssi($urlp,%form);
	    if ($urlp=~/\.page$/) {
		($texversion,my $number_of_columns_page) = &page_cleanup($texversion);
		if ($number_of_columns_page > $number_of_columns) {$number_of_columns=$number_of_columns_page;} 
		$texversion =~ s/\\end{document}\d*/\\end{document}/;
		$flag_page_in_sequence = 'YES';
	    } 
	    my $lonidsdir=$r->dir_config('lonIDsDir');
	    my $envfile=$ENV{'user.environment'};
	    $envfile=~/\/([^\/]+)\.id$/;
            $envfile=$1;
	    &Apache::lonnet::transfer_profile_to_env($lonidsdir,$envfile);
	    my $current_counter=$ENV{'form.counter'};
	    if(($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'no') ||
	       ($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'only')) {
		my %form;
		$form{'grade_target'}='answer';
		$form{'answer_output_mode'}='tex';
		my $answer=&Apache::lonnet::ssi($urlp,%form);
                &Apache::lonnet::appenv(('form.counter' => $current_counter));
		if ($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'no') {
		    $texversion=~s/(\\keephidden{ENDOFPROBLEM})/$answer$1/;
		} else {
		    if ($urlp=~/\.(problem|exam|quiz|library)$/) {
			$texversion=&print_latex_header($helper->{'VARS'}->{'LATEX_TYPE'});
			$texversion.='\vskip 0 mm \noindent\textbf{'.&Apache::lonnet::gettitle($urlp).'}\vskip 0 mm ';
			$texversion.=&path_to_problem ($urlp,$LaTeXwidth);
			$texversion.='\vskip 1 mm '.$answer;
		    } else {
			$texversion='';
		    }
		}
	    }
	    if ($flag_latex_header_remove ne 'NO') {
		$texversion = &latex_header_footer_remove($texversion);
	    } else {
		$texversion =~ s/\\end{document}//;
	    }
	    if ($helper->{'VARS'}->{'TABLE_INDEX'} eq 'yes') {
		$texversion=&IndexCreation($texversion,$urlp);
	    }
	    if (($selectionmade == 4) and ($assignment ne $prevassignment)) {
		my $name = &get_name();
		my $courseidinfo = &get_course();
		if (defined($courseidinfo)) { $courseidinfo=' - '.$courseidinfo }
		$prevassignment=$assignment;
		$result .='\newpage \noindent\parbox{\minipagewidth}{\noindent\\lhead{\\textit{\\textbf{'.$name.'}}'.$courseidinfo.' \\hfill \\thepage \\\\ \\textit{'.$assignment.'}}} \vskip -5 mm ';
	    }
	    $result .= $texversion;
	    $flag_latex_header_remove = 'YES';   
	}
	&Apache::lonnet::delenv('form.counter');
	if ($flag_page_in_sequence eq 'YES') {$result =~ s/\\usepackage{calc}/\\usepackage{calc}\\usepackage{longtable}/;}	
	$result .= '\end{document}';
     } elsif (($helper->{'VARS'}->{'PRINT_TYPE'} eq 'problems_for_students') || ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'resources_for_students')) { 
     #-- prints assignments for whole class or for selected students  
	 if ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'problems_for_students') {
	     $selectionmade=5;
	 } elsif ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'resources_for_students') {
	     $selectionmade=8;
	 }
	 my @students=split /\|\|\|/, $helper->{'VARS'}->{'STUDENTS'};
	 my @master_seq=split /\|\|\|/, $helper->{'VARS'}->{'RESOURCES'};
	 #loop over students
	 my $flag_latex_header_remove = 'NO'; 
	 my %moreenv;
	 $moreenv{'form.textwidth'}=$LaTeXwidth;
	 &Apache::lonnet::appenv(%moreenv);
	 my %prog_state=&Apache::lonhtmlcommon::Create_PrgWin($r,'Print Status','Class Print Status',$#students+1);
	 foreach my $person (@students) {
	    my $current_output = ''; 
#	    my ($usersection,$username,$userdomain) = split /:/,$person;
	    my ($username,$userdomain) = split /:/,$person;
	    my $fullname = &get_name($username,$userdomain);

	    if ($parmhash{'anonymous_quiz'}=~/yes/) {$fullname=' ';} #put here something reasonable for anonymous exams or quiz
            #goes through all resources, checks if they are available for current student, and produces output   
	    &Apache::lonnet::delenv('form.counter');
	    &Apache::lonxml::init_counter();
	    foreach my $curresline (@master_seq)  {
		if ((($curresline=~ m/\.(problem|exam|quiz|assess|survey|form|library)$/) && ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'problems_for_students')) || ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'resources_for_students')) {
		    my ($map,$id,$res_url) = &Apache::lonnet::decode_symb($curresline);
		    if (&Apache::lonnet::allowed('bre',$res_url)) {
			my $rendered = &Apache::loncommon::get_student_view($curresline,$username,$userdomain,
                                                                        $ENV{'request.course.id'},'tex');
			my $lonidsdir=$r->dir_config('lonIDsDir');
			my $envfile=$ENV{'user.environment'};
			$envfile=~/\/([^\/]+)\.id$/;
			$envfile=$1;
			&Apache::lonnet::transfer_profile_to_env($lonidsdir,$envfile);
			my $current_counter=$ENV{'form.counter'};
			if(($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'no') ||
			   ($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'only')) {
			    my %form;
			    $form{'answer_output_mode'}='tex';
			    $form{'latex_type'}=$helper->{'VARS'}->{'LATEX_TYPE'};
			    my $ansrendered = &Apache::loncommon::get_student_answers($curresline,$username,$userdomain,$ENV{'request.course.id'},%form);
			    &Apache::lonnet::appenv(('form.counter' => $current_counter));
			    if ($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'no') {
				$rendered=~s/(\\keephidden{ENDOFPROBLEM})/$ansrendered$1/;
			    } else {
				$rendered=&print_latex_header($helper->{'VARS'}->{'LATEX_TYPE'});
				$rendered.='\vskip 0 mm \noindent\textbf{'.&Apache::lonnet::gettitle($curresline).'}\vskip 0 mm ';
				$rendered.=&path_to_problem ($curresline,$LaTeXwidth);
				$rendered.='\vskip 1 mm '.$ansrendered;
			    }
			}
			if ($flag_latex_header_remove eq 'YES') {
			    $rendered = &latex_header_footer_remove($rendered);
			} else {
			    $rendered =~ s/\\end{document}//;
			}
			$current_output .= $rendered;
		    }
		    $flag_latex_header_remove = 'YES';
		}
	    }
	    my $courseidinfo = &get_course();
	    if (defined($courseidinfo)) { $courseidinfo=' - '.$courseidinfo }
	    my $currentassignment=&Apache::lonxml::latex_special_symbols($helper->{VARS}->{'assignment'},'header');
	    if ($current_output=~/\\documentclass/) {
		$current_output =~ s/\\begin{document}/\\setlength{\\topmargin}{1cm} \\begin{document}\\noindent\\lhead{\\textit{\\textbf{$fullname}}$courseidinfo \\hfill \\thepage \\\\ \\textit{$currentassignment}}\\vskip 3 mm /;
	    } else {
		my $blanspages = '';
		for (my $j=0;$j<$helper->{'VARS'}->{'EMPTY_PAGES'};$j++) {$blanspages.='\clearpage\strut\clearpage';}
		$current_output = '\strut\\newline\\noindent\\makebox[\\textwidth/$number_of_columns][b]{\\hrulefill}\\newline\\noindent{\\tiny Printed from LON-CAPA\\copyright MSU{\\hfill} Licensed under GNU General Public License }\\newpage '.$blanspages.'\setcounter{page}{1}\noindent\parbox{\minipagewidth}{\noindent\\lhead{\\textit{\\textbf{'.$fullname.'}}'.$courseidinfo.' \\hfill \\thepage \\\\ \\textit{'.$currentassignment.'}}} \vskip -5 mm '.$current_output;
	    }
	    $result .= $current_output;
	    &Apache::lonnet::delenv('form.counter');	    
	    &Apache::lonxml::init_counter(); 
	    &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,
					     &mt('last student').' '.$fullname);
	}
	&Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
	$result .= '\end{document}';
	&Apache::lonnet::delenv('form.textwidth');
    } elsif ($helper->{'VARS'}->{'PRINT_TYPE'} eq 'problems_from_directory') {      
    #prints selected problems from the subdirectory 
	$selectionmade = 6;
        my @list_of_files=split /\|\|\|/, $helper->{'VARS'}->{'FILES'};
	@list_of_files=sort @list_of_files;
	my $flag_latex_header_remove = 'NO'; 
	my $rndseed=time;
	if ($helper->{'VARS'}->{'curseed'}) {
	    $rndseed=$helper->{'VARS'}->{'curseed'};
	}
	for (my $i=0;$i<=$#list_of_files;$i++) {
	    my $urlp = $list_of_files[$i];
	    $urlp=~s|//|/|;
	    if ($urlp=~/\//) {
		my %form;
		$form{'grade_target'}='tex';
		$form{'textwidth'}=$LaTeXwidth;
		$form{'rndseed'}=$rndseed;
		if ($urlp =~ m|/home/([^/]+)/public_html|) {
		    $urlp =~ s|/home/([^/]*)/public_html|/~$1|;
		} else {
		    $urlp =~ s|^/home/httpd/html||;
		}
		my $texversion=&Apache::lonnet::ssi($urlp,%form);
		if(($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'no') ||
		   ($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'only')) {
		    my %form;
		    $form{'grade_target'}='answer';
		    $form{'answer_output_mode'}='tex';
		    $form{'latex_type'}=$helper->{'VARS'}->{'LATEX_TYPE'};
		    $form{'rndseed'}=$rndseed;
		    my $answer=&Apache::lonnet::ssi($urlp,%form);
		    if ($helper->{'VARS'}->{'ANSWER_TYPE'} eq 'no') {
			$texversion=~s/(\\keephidden{ENDOFPROBLEM})/$answer$1/;
		    } else {
			$texversion=&print_latex_header($helper->{'VARS'}->{'LATEX_TYPE'});
			if ($helper->{'VARS'}->{'construction'} ne '1') {
			    $texversion.='\vskip 0 mm \noindent ';
			    $texversion.=&path_to_problem ($urlp,$LaTeXwidth);
			} else {
			    $texversion.='\vskip 0 mm \noindent\textbf{Prints from construction space - there is no title.}\vskip 0 mm ';
			    my $URLpath=$urlp;
			    $URLpath=~s/~([^\/]+)/public_html\/$1\/$1/;
			    $texversion.=&path_to_problem ($URLpath,$LaTeXwidth);
			}
			$texversion.='\vskip 1 mm '.$answer.'\end{document}';
		    }
		}
                #this chunck is responsible for printing the path to problem
		my $newurlp=$urlp;
		if ($newurlp=~/~/) {$newurlp=~s|\/~([^\/]+)\/|\/home\/$1\/public_html\/|;}
		$newurlp=&path_to_problem($newurlp,$LaTeXwidth);
		$texversion =~ s/(\\begin{minipage}{\\textwidth})/$1 $newurlp/;
		if ($flag_latex_header_remove ne 'NO') {
		    $texversion = &latex_header_footer_remove($texversion);
		} else {
		    $texversion =~ s/\\end{document}//;
		}
		if ($helper->{'VARS'}->{'TABLE_INDEX'} eq 'yes') {
		    $texversion=&IndexCreation($texversion,$urlp);
		}
		if ($helper->{'VARS'}->{'CONSTR_RESOURSE_URL'} eq 'yes') {
		    $texversion=~s/(\\addcontentsline\{toc\}\{subsection\}\{[^\}]*\})/$1 URL: \\verb|$urlp| \\strut\\\\\\strut /;
		    
		}
		$result .= $texversion;
	    }
	    $flag_latex_header_remove = 'YES';  
	}
	if ($helper->{VARS}->{'construction'} eq '1') {$result=~s/(\\typeout)/ RANDOM SEED IS $rndseed $1/;}
	$result .= '\end{document}';      	
    }
#-------------------------------------------------------- corrections for the different page formats
    $result = &page_format_transformation($papersize,$laystyle,$numberofcolumns,$helper->{'VARS'}->{'PRINT_TYPE'},$result,$helper->{VARS}->{'assignment'},$helper->{'VARS'}->{'TABLE_CONTENTS'},$helper->{'VARS'}->{'TABLE_INDEX'});
    $result = &latex_corrections($number_of_columns,$result);
    #changes page's parameters for the one column output 
    if ($numberofcolumns == 1) {
	$result =~ s/\\textwidth\s*=\s*\d*\.?\d*\s*(cm|mm|in)/\\textwidth= $helper->{'VARS'}->{'pagesize.width'} $helper->{'VARS'}->{'pagesize.widthunit'} /;
	$result =~ s/\\textheight\s*=\s*\d*\.?\d*\s*(cm|mm|in)/\\textheight $helper->{'VARS'}->{'pagesize.height'} $helper->{'VARS'}->{'pagesize.heightunit'} /;
	$result =~ s/\\evensidemargin\s*=\s*\d*\.?\d*\s*(cm|mm|in)/\\evensidemargin= $helper->{'VARS'}->{'pagesize.lmargin'} $helper->{'VARS'}->{'pagesize.lmarginunit'} /;
	$result =~ s/\\oddsidemargin\s*=\s*\d*\.?\d*\s*(cm|mm|in)/\\oddsidemargin= $helper->{'VARS'}->{'pagesize.lmargin'} $helper->{'VARS'}->{'pagesize.lmarginunit'} /;
    }
#-- writing .tex file in prtspool 
    my $temp_file;
    my $filename = "/home/httpd/prtspool/$ENV{'user.name'}_$ENV{'user.domain'}_printout_".time."_".rand(10000000).".tex";
    unless ($temp_file = Apache::File->new('>'.$filename)) {
	$r->log_error("Couldn't open $filename for output $!");
	return SERVER_ERROR; 
    }

    my $identifier = time.'_'.int(rand(1000));
    &Apache::lonnet::appenv('cgi.'.$identifier.'.file'   => $filename,
                            'cgi.'.$identifier.'.layout'  => $laystyle,
                            'cgi.'.$identifier.'.numcol'  => $numberofcolumns,
                            'cgi.'.$identifier.'.selection' => $selectionmade,
			    'cgi.'.$identifier.'tableofcontents' => $helper->{'VARS'}->{'TABLE_CONTENTS'},
			    'cgi.'.$identifier.'tableofindex' => $helper->{'VARS'}->{'TABLE_INDEX'},
			    'cgi.'.$identifier.'role' => $ENV{'request.role.adv'});
 
    print $temp_file $result;

$r->print(<<FINALEND);
<meta http-equiv="Refresh" content="0; url=/cgi-bin/printout.pl?$identifier">
</body>
</html>
FINALEND
}



sub handler {

    my $r = shift;
    my $helper;

#    my $loaderror=&Apache::lonnet::overloaderror($r);
#    if ($loaderror) { return $loaderror; }
#    $loaderror=
#       &Apache::lonnet::overloaderror($r,
#         $ENV{'course.'.$ENV{'request.course.id'}.'.home'});
#    if ($loaderror) { return $loaderror; }

    my $result = printHelper($r);
    if (!ref($result)) {
	return $result;
    }
    $helper = $result;
   
#	my $key; 
#	foreach $key (keys %{$helper->{'VARS'}}) {
#	    $r->print(' '.$key.'->'.$helper->{'VARS'}->{$key}.'<-<br />');
#	}
#	foreach $key (keys %ENV) {
#	    $r->print(' '.$key.'->'.$ENV{$key}.'<-<br />');
#	}
#	return OK;

    my %parmhash=&Apache::lonnet::coursedescription($ENV{'request.course.id'});
    
#	my $key; 
#	foreach $key (keys %parmhash) {
#	    $r->print(' '.$key.'->'.$parmhash{$key}.'<-<br />');
#	}
#	return OK;

    &output_data($r,$helper,\%parmhash);
    return OK;
} 

use Apache::lonhelper;

sub addMessage {
    my $text = shift;
    my $paramHash = Apache::lonhelper::getParamHash();
    $paramHash->{MESSAGE_TEXT} = $text;
    Apache::lonhelper::message->new();
}

use Data::Dumper;

sub printHelper {
    my $r = shift;

    if ($ENV{'request.course.id'}) {
	my $fn=$ENV{'request.course.fn'};
	tie(%hash,'GDBM_File',"$fn.db",&GDBM_READER(),0640);
    }

    if ($r->header_only) {
        if ($ENV{'browser.mathml'}) {
            &Apache::loncommon::content_type($r,'text/xml');
        } else {
            &Apache::loncommon::content_type($r,'text/html');
        }
        $r->send_http_header;
        return OK;
    }

    # Send header, nocache
    if ($ENV{'browser.mathml'}) {
        &Apache::loncommon::content_type($r,'text/xml');
    } else {
        &Apache::loncommon::content_type($r,'text/html');
    }
    &Apache::loncommon::no_cache($r);
    $r->send_http_header;
    $r->rflush();

    # Unfortunately, this helper is so complicated we have to
    # write it by hand

    Apache::loncommon::get_unprocessed_cgi($ENV{QUERY_STRING});
    
    my $helper = Apache::lonhelper::helper->new("Printing Helper");
    $helper->declareVar('symb');
    $helper->declareVar('postdata');    
    $helper->declareVar('curseed');    
    $helper->declareVar('filename');
    $helper->declareVar('construction');
    $helper->declareVar('assignment');
    $helper->declareVar('style_file');
    
    # This will persistently load in the data we want from the
    # very first screen.
    # Detect whether we're coming from construction space
    if ($ENV{'form.postdata'}=~/^(?:http:\/\/[^\/]+\/|\/|)\~([^\/]+)\/(.*)$/) {
        $helper->{VARS}->{'filename'} = "~$1/$2";
        $helper->{VARS}->{'construction'} = 1;
    } else {
        if ($ENV{'form.postdata'}) {
            $helper->{VARS}->{'symb'} = &Apache::lonnet::symbread($ENV{'form.postdata'});
        }
        if ($ENV{'form.symb'}) {
            $helper->{VARS}->{'symb'} = $ENV{'form.symb'};
        }
        if ($ENV{'form.url'}) {
            $helper->{VARS}->{'symb'} = &Apache::lonnet::symbread($helper->{VARS}->{'postdata'});
        }
    }

    if ($ENV{'form.symb'}) {
        $helper->{VARS}->{'symb'} = $ENV{'form.symb'};
    }
    if ($ENV{'form.url'}) {
        $helper->{VARS}->{'symb'} = &Apache::lonnet::symbread($helper->{VARS}->{'postdata'});

    }
    
    my ($resourceTitle,$sequenceTitle,$mapTitle) = &details_for_menu;
    if ($sequenceTitle ne '') {$helper->{VARS}->{'assignment'}=$sequenceTitle;}

    
    # Extract map
    my $symb = $helper->{VARS}->{'symb'};
    my ($map, $id, $url);
    my $subdir;

    # Get the resource name from construction space
    if ($helper->{VARS}->{'construction'}) {
        $resourceTitle = substr($helper->{VARS}->{'filename'}, 
                                rindex($helper->{VARS}->{'filename'}, '/')+1);
        $subdir = substr($helper->{VARS}->{'filename'},
                         0, rindex($helper->{VARS}->{'filename'}, '/') + 1);
    } else {
        ($map, $id, $url) = &Apache::lonnet::decode_symb($symb);
        $helper->{VARS}->{'postdata'} = Apache::lonnet::clutter($url);

        if (!$resourceTitle) { # if the resource doesn't have a title, use the filename
            my $postdata = $helper->{VARS}->{'postdata'};
            $resourceTitle = substr($postdata, rindex($postdata, '/') + 1);
        }
        $subdir = &Apache::lonnet::filelocation("", $url);
    }
    if (!$helper->{VARS}->{'curseed'} && $ENV{'form.curseed'}) {
	$helper->{VARS}->{'curseed'}=$ENV{'form.curseed'};
    }

    my $userCanSeeHidden = Apache::lonnavmaps::advancedUser();
    my $userPriviledged = ($ENV{'request.role'}=~m/^cc\./ or
			   $ENV{'request.role'}=~m/^in\./ or
			   $ENV{'request.role'}=~m/^ta\./);

    Apache::lonhelper::registerHelperTags();

    # "Delete everything after the last slash."
    $subdir =~ s|/[^/]+$||;
    if (not $helper->{VARS}->{'construction'}) {
	$subdir='/home/httpd/html/res/'.$subdir;
    }
    # "Remove all duplicate slashes."
    $subdir =~ s|/+|/|g;

    # What can be printed is a very dynamic decision based on
    # lots of factors. So we need to dynamically build this list.
    # To prevent security leaks, states are only added to the wizard
    # if they can be reached, which ensures manipulating the form input
    # won't allow anyone to reach states they shouldn't have permission
    # to reach.

    # printChoices is tracking the kind of printing the user can
    # do, and will be used in a choices construction later.
    # In the meantime we will be adding states and elements to
    # the helper by hand.
    my $printChoices = [];
    my $paramHash;

    if ($resourceTitle) {
        push @{$printChoices}, ["<b><i>$resourceTitle</i></b> (what you just saw on the screen)", 'current_document', 'PAGESIZE'];
    }

#    $r->print($helper->{VARS}->{'postdata'});

    # Useful filter strings
    my $isProblem = '$res->is_problem()';
    $isProblem .= ' && !$res->randomout()' if !$userCanSeeHidden;
    my $isProblemOrMap = '$res->is_problem() || $res->is_map()';
    my $isNotMap = '!$res->is_map()';
    $isNotMap .= ' && !$res->randomout()' if !$userCanSeeHidden;
    my $isMap = '$res->is_map()';
    my $symbFilter = '$res->symb()';
    my $urlValue = '$res->src()';

    $helper->declareVar('SEQUENCE');

    # Useful for debugging: Dump the help vars
    #$r->print(Dumper($helper->{VARS}));
    #$r->print($map);

    # If we're in a sequence...
    if (($helper->{'VARS'}->{'construction'} ne '1') &&
	$helper->{VARS}->{'postdata'} &&
	$helper->{VARS}->{'assignment'}) {
        # Allow problems from sequence
        push @{$printChoices}, ["<b>Problems</b> in <b><i>$sequenceTitle</i></b>", 'map_problems', 'CHOOSE_PROBLEMS'];
        # Allow all resources from sequence
        push @{$printChoices}, ["<b>Resources</b> in <b><i>$sequenceTitle</i></b>", 'map_problems_pages', 'CHOOSE_PROBLEMS_HTML'];

        my $helperFragment = <<HELPERFRAGMENT;
  <state name="CHOOSE_PROBLEMS" title="Select Problem(s) to print">
    <message>(mark them then click "next" button) <br /></message>
    <resource variable="RESOURCES" multichoice="1" toponly='1' addstatus="1">
      <nextstate>PAGESIZE</nextstate>
      <filterfunc>return $isProblem;</filterfunc>
      <mapurl>$map</mapurl>
      <valuefunc>return $symbFilter;</valuefunc>
      </resource>
    </state>

  <state name="CHOOSE_PROBLEMS_HTML" title="Select Resource(s) to print">
    <message>(mark them then click "next" button) <br /></message>
    <resource variable="RESOURCES" multichoice="1" toponly='1' addstatus="1">
      <nextstate>PAGESIZE</nextstate>
      <filterfunc>return $isNotMap;</filterfunc>
      <mapurl>$map</mapurl>
      <valuefunc>return $symbFilter;</valuefunc>
      </resource>
    </state>
HELPERFRAGMENT

        &Apache::lonxml::xmlparse($r, 'helper', $helperFragment); 
    }

    # If the user is priviledged, allow them to print all 
    # problems in the course, optionally for selected students
    if ($userPriviledged && ($helper->{VARS}->{'postdata'}=~/\/res\//)) { 
        push @{$printChoices}, ['<b>Problems</b> from <b>entire course</b>', 'all_problems', 'ALL_PROBLEMS'];
	if ($helper->{VARS}->{'assignment'}) {
	    push @{$printChoices}, ["<b>Problems</b> from <b><i>$sequenceTitle</i></b> for <b>selected students</b>", 'problems_for_students', 'CHOOSE_STUDENTS'];
	}

        &Apache::lonxml::xmlparse($r, 'helper', <<CHOOSE_STUDENTS);
  <state name="ALL_PROBLEMS" title="Select Problem(s) to print">
    <message>(mark them then click "next" button) <br /></message>
    <resource variable="RESOURCES" multichoice="1"
	suppressEmptySequences='1' addstatus="1">
      <nextstate>PAGESIZE</nextstate>
      <filterfunc>return $isProblemOrMap;</filterfunc>
      <choicefunc>return $isProblem;</choicefunc>
      <valuefunc>return $symbFilter;</valuefunc>
      </resource>
    </state>

  <state name="CHOOSE_STUDENTS" title="Select Students and Resources">
    <student multichoice='1' variable="STUDENTS" nextstate="PAGESIZE" />
    <message><br /><big><i><b>Select resources for the assignment</b></i></big><br /></message>
    <resource variable="RESOURCES" multichoice="1" addstatus="1">
      <filterfunc>return $isProblem;</filterfunc>
      <mapurl>$map</mapurl>
      <valuefunc>return $symbFilter;</valuefunc>
      </resource>
    <message><br /><big><i><b>How should the results be printed?</b></i></big><br /></message>
    <choices variable="EMPTY_PAGES">
      <choice computer='0'>Start each student\'s assignment on a new page/column (add a pagefeed after each assignment)</choice>
      <choice computer='1'>Add one empty page/column after each student\'s assignment</choice>
      <choice computer='2'>Add two empty pages/column after each student\'s assignment</choice>
      <choice computer='3'>Add three empty pages/column after each student\'s assignment</choice>
      </choices>
    </state>
CHOOSE_STUDENTS

	if ($helper->{VARS}->{'assignment'}) {
	    push @{$printChoices}, ["<b>Resources</b> from <b><i>$sequenceTitle</i></b> for <b>selected students</b>", 'resources_for_students', 'CHOOSE_STUDENTS1'];
	}
        &Apache::lonxml::xmlparse($r, 'helper', <<CHOOSE_STUDENTS1);
  <state name="CHOOSE_STUDENTS1" title="Select Students and Resources">
    <student multichoice='1' variable="STUDENTS" nextstate="PAGESIZE" />
    <message><br /><big><i><b>Select resources for the assignment</b></i></big><br /></message>
    <resource variable="RESOURCES" multichoice="1" addstatus="1">
      <filterfunc>return $isNotMap;</filterfunc>
      <mapurl>$map</mapurl>
      <valuefunc>return $symbFilter;</valuefunc>
      </resource>
    <message><br /><big><i><b>How should the results be printed?</b></i></big><br /></message>
    <choices variable="EMPTY_PAGES">
      <choice computer='0'>Start each student\'s assignment on a new page/column (add a pagefeed after each assignment)</choice>
      <choice computer='1'>Add one empty page/column after each student\'s assignment</choice>
      <choice computer='2'>Add two empty pages/column after each student\'s assignment</choice>
      <choice computer='3'>Add three empty pages/column after each student\'s assignment</choice>
      </choices>
    </state>
CHOOSE_STUDENTS1

    }

    # FIXME: That RE should come from a library somewhere.
    if ((((&Apache::lonnet::allowed('bre',$subdir) eq 'F') and ($helper->{VARS}->{'postdata'}=~/\.(problem|exam|quiz|assess|survey|form|library|page|xml|html|htm|xhtml|xhtm)/)) or defined $helper->{'VARS'}->{'construction'}) and $ENV{'request.role.adv'} and $subdir ne '/home/httpd/html/res/') {    
        push @{$printChoices}, ["<b>Problems</b> from current subdirectory <b><i>$subdir</i></b>", 'problems_from_directory', 'CHOOSE_FROM_SUBDIR'];

        my $f = '$filename';
        my $xmlfrag = <<CHOOSE_FROM_SUBDIR;
  <state name="CHOOSE_FROM_SUBDIR" title="Select File(s) from <b><small>$subdir</small></b> to print">
    <message>(mark them then click "next" button) <br /></message>
    <files variable="FILES" multichoice='1'>
      <nextstate>PAGESIZE</nextstate>
      <filechoice>return '$subdir';</filechoice>
CHOOSE_FROM_SUBDIR
        
        # this is broken up because I really want interpolation above,
        # and I really DON'T want it below
        $xmlfrag .= <<'CHOOSE_FROM_SUBDIR';
      <filefilter>return Apache::lonhelper::files::not_old_version($filename) &&
	  $filename =~ m/\.(problem|exam|quiz|assess|survey|form|library)$/;
      </filefilter>
      </files>
    </state>
CHOOSE_FROM_SUBDIR
        &Apache::lonxml::xmlparse($r, 'helper', $xmlfrag);
    }

    # Allow the user to select any sequence in the course, feed it to
    # another resource selector for that sequence
    if (!$helper->{VARS}->{'construction'}) {
	push @$printChoices, ["<b>Resources</b> from <b>selected sequence</b> in course",
			      'select_sequences', 'CHOOSE_SEQUENCE'];
	my $escapedSequenceName = $helper->{VARS}->{'SEQUENCE'};
	#Escape apostrophes and backslashes for Perl
	$escapedSequenceName =~ s/\\/\\\\/g;
	$escapedSequenceName =~ s/'/\\'/g;
	&Apache::lonxml::xmlparse($r, 'helper', <<CHOOSE_FROM_ANY_SEQUENCE);
  <state name="CHOOSE_SEQUENCE" title="Select Sequence To Print From">
    <message>Select the sequence to print resources from:</message>
    <resource variable="SEQUENCE">
      <nextstate>CHOOSE_FROM_ANY_SEQUENCE</nextstate>
      <filterfunc>return \$res->is_sequence;</filterfunc>
      <valuefunc>return $urlValue;</valuefunc>
      </resource>
    </state>
  <state name="CHOOSE_FROM_ANY_SEQUENCE" title="Select Resources To Print">
    <message>(mark desired resources then click "next" button) <br /></message>
    <resource variable="RESOURCES" multichoice="1" toponly='1' addstatus="1">
      <nextstate>PAGESIZE</nextstate>
      <filterfunc>return $isProblem</filterfunc>
      <mapurl evaluate='1'>return '$escapedSequenceName';</mapurl>
      <valuefunc>return $symbFilter;</valuefunc>
      </resource>
    </state>
CHOOSE_FROM_ANY_SEQUENCE
}

    # Generate the first state, to select which resources get printed.
    Apache::lonhelper::state->new("START", "Select Printing Options:");
    $paramHash = Apache::lonhelper::getParamHash();
    $paramHash->{MESSAGE_TEXT} = "";
    Apache::lonhelper::message->new();
    $paramHash = Apache::lonhelper::getParamHash();
    $paramHash->{'variable'} = 'PRINT_TYPE';
    $helper->declareVar('PRINT_TYPE');
    $paramHash->{CHOICES} = $printChoices;
    Apache::lonhelper::choices->new();

    my $startedTable = 0; # have we started an HTML table yet? (need
                          # to close it later)

    if (($ENV{'request.role.adv'} and &Apache::lonnet::allowed('vgr',$ENV{'request.course.id'})) or 
	($helper->{VARS}->{'construction'} eq '1')) {
	addMessage("<hr width='33%' /><table><tr><td align='right'>Print: </td><td>");
        $paramHash = Apache::lonhelper::getParamHash();
	$paramHash->{'variable'} = 'ANSWER_TYPE';   
	$helper->declareVar('ANSWER_TYPE');         
        $paramHash->{CHOICES} = [
                                   ['Without Answers', 'yes'],
                                   ['With Answers', 'no'],
                                   ['Only Answers', 'only'] ];
        Apache::lonhelper::dropdown->new();
	addMessage("</td></tr>");
	$startedTable = 1;
    }

    if ($ENV{'request.role.adv'}) {
	if (!$startedTable) {
	    addMessage("<hr width='33%' /><table><tr><td align='right'>LaTeX mode: </td><td>");
	    $startedTable = 1;
	} else {
	    addMessage("<tr><td align='right'>LaTeX mode: </td><td>");
	}
        $paramHash = Apache::lonhelper::getParamHash();
	$paramHash->{'variable'} = 'LATEX_TYPE';   
	$helper->declareVar('LATEX_TYPE');  
	if ($helper->{VARS}->{'construction'} eq '1') {       
	    $paramHash->{CHOICES} = [
				     ['standard LaTeX mode', 'standard'], 
				     ['LaTeX batchmode', 'batchmode'], ];
	} else {
	    $paramHash->{CHOICES} = [
				     ['LaTeX batchmode', 'batchmode'],
				     ['standard LaTeX mode', 'standard'] ];
	}
        Apache::lonhelper::dropdown->new();
 
	addMessage("</td></tr><tr><td align='right'>Print Table of Contents: </td><td>");
        $paramHash = Apache::lonhelper::getParamHash();
	$paramHash->{'variable'} = 'TABLE_CONTENTS';   
	$helper->declareVar('TABLE_CONTENTS');         
        $paramHash->{CHOICES} = [
                                   ['No', 'no'],
                                   ['Yes', 'yes'] ];
        Apache::lonhelper::dropdown->new();
	addMessage("</td></tr>");
        
	if (not $helper->{VARS}->{'construction'}) {
	    addMessage("<tr><td align='right'>Print Index: </td><td>");
	    $paramHash = Apache::lonhelper::getParamHash();
	    $paramHash->{'variable'} = 'TABLE_INDEX';   
	    $helper->declareVar('TABLE_INDEX');         
	    $paramHash->{CHOICES} = [
				     ['No', 'no'],
				     ['Yes', 'yes'] ];
	    Apache::lonhelper::dropdown->new();
	    addMessage("</td></tr>");
	}

	if ($helper->{'VARS'}->{'construction'}) { 
	    my $stylevalue=$ENV{'construct.style'};
	    my $xmlfrag .= <<"RNDSEED";
	    <message><tr><td align='right'>Use random seed: </td><td></message>
	    <string variable="curseed" size="15" maxlength="15">
		<defaultvalue>
	            return $helper->{VARS}->{'curseed'};
	        </defaultvalue>
	    </string>
	     <message></td></tr><tr><td align="right">Use style file:</td><td></message>
             <message><input type="text" size="40" name="style_file_value" value="$stylevalue"  />&nbsp; <a href="javascript:openbrowser('helpform','style_file','sty')">Select style file</a> </td><td></message>
             <message></td></tr></message>
RNDSEED
            &Apache::lonxml::xmlparse($r, 'helper', $xmlfrag);
	    $helper->{'VARS'}->{'style_file'}=$ENV{'form.style_file_value'};
	}
    }




    if ($startedTable) {
	addMessage("</table>");
    }

    Apache::lonprintout::page_format_state->new("FORMAT");

    # Generate the PAGESIZE state which will offer the user the margin
    # choices if they select one column
    Apache::lonhelper::state->new("PAGESIZE", "Set Margins");
    Apache::lonprintout::page_size_state->new('pagesize', 'FORMAT', 'FINAL');


    $helper->process();

    # MANUAL BAILOUT CONDITION:
    # If we're in the "final" state, bailout and return to handler
    if ($helper->{STATE} eq 'FINAL') {
        return $helper;
    }    

    $r->print($helper->display());

    Apache::lonhelper::unregisterHelperTags();

    untie %hash;

    return OK;
}


1;

package Apache::lonprintout::page_format_state;

=pod

=head1 Helper element: page_format_state

See lonhelper.pm documentation for discussion of the helper framework.

Apache::lonprintout::page_format_state is an element that gives the 
user an opportunity to select the page layout they wish to print 
with: Number of columns, portrait/landscape, and paper size. If you 
want to change the paper size choices, change the @paperSize array 
contents in this package.

page_format_state is always directly invoked in lonprintout.pm, so there
is no tag interface. You actually pass parameters to the constructor.

=over 4

=item * B<new>(varName): varName is where the print information will be stored in the format FIXME.

=back

=cut

use Apache::lonhelper;

no strict;
@ISA = ("Apache::lonhelper::element");
use strict;

my $maxColumns = 2;
my @paperSize = ("Letter [8 1/2x11 in]", "Legal [8 1/2x14 in]", 
                 "Ledger/Tabloid [11x17 in]", "Executive [7 1/2x10 in]",
                 "A2 [420x594 mm]", "A3 [297x420 mm]", "A4 [210x297 mm]", 
                 "A5 [148x210 mm]", "A6 [105x148 mm]" );

# Tentative format: Orientation (L = Landscape, P = portrait) | Colnum |
#                   Paper type

sub new { 
    my $self = Apache::lonhelper::element->new();

    shift;

    $self->{'variable'} = shift;
    my $helper = Apache::lonhelper::getHelper();
    $helper->declareVar($self->{'variable'});
    bless($self);
    return $self;
}

sub render {
    my $self = shift;
    my $helper = Apache::lonhelper::getHelper();
    my $result = '';
    my $var = $self->{'variable'};

    $result .= <<STATEHTML;

<hr width="33%" />
<table cellpadding="3">
  <tr>
    <td align="center"><b>Page layout</b></td>
    <td align="center"><b>Number of columns</b></td>
    <td align="center"><b>Paper type</b></td>
  </tr>
  <tr>
    <td>
      <input type="radio" name="${var}.layout" value="L" /> Landscape<br />
      <input type="radio" name="${var}.layout" value="P" checked='1'  /> Portrait
    </td>
    <td align="center">
      <select name="${var}.cols">
STATEHTML

    my $i;
    for ($i = 1; $i <= $maxColumns; $i++) {
        if ($i == 2) {
            $result .= "<option value='$i' selected>$i</option>\n";
        } else {
            $result .= "<option value='$i'>$i</option>\n";
        }
    }

    $result .= "</select></td><td>\n";
    $result .= "<select name='${var}.paper'>\n";

    $i = 0;
    foreach (@paperSize) {
        if ($i == 0) {
            $result .= "<option selected value='$i'>" . $paperSize[$i] . "</option>\n";
        } else {
            $result .= "<option value='$i'>" . $paperSize[$i] . "</option>\n";
        }
        $i++;
    }

    $result .= "</select></td></tr></table>";
    return $result;
}

sub postprocess {
    my $self = shift;

    my $var = $self->{'variable'};
    my $helper = Apache::lonhelper->getHelper();
    $helper->{VARS}->{$var} = 
        $ENV{"form.$var.layout"} . '|' . $ENV{"form.$var.cols"} . '|' .
        $ENV{"form.$var.paper"};
    return 1;
}

1;

package Apache::lonprintout::page_size_state;

=pod

=head1 Helper element: page_size_state

See lonhelper.pm documentation for discussion of the helper framework.

Apache::lonprintout::page_size_state is an element that gives the 
user the opportunity to further refine the page settings if they
select a single-column page.

page_size_state is always directly invoked in lonprintout.pm, so there
is no tag interface. You actually pass parameters to the constructor.

=over 4

=item * B<new>(varName): varName is where the print information will be stored in the format FIXME.

=back

=cut

use Apache::lonhelper;

no strict;
@ISA = ("Apache::lonhelper::element");
use strict;



sub new { 
    my $self = Apache::lonhelper::element->new();

    shift; # disturbs me (probably prevents subclassing) but works (drops
           # package descriptor)... - Jeremy

    $self->{'variable'} = shift;
    my $helper = Apache::lonhelper::getHelper();
    $helper->declareVar($self->{'variable'});

    # The variable name of the format element, so we can look into 
    # $helper->{VARS} to figure out whether the columns are one or two
    $self->{'formatvar'} = shift;

    # The state to transition to after selection, or after discovering
    # the cols are not set to 1
    $self->{NEXTSTATE} = shift;
    bless($self);
    return $self;
}

sub render {
    my $self = shift;
    my $helper = Apache::lonhelper::getHelper();
    my $result = '';
    my $var = $self->{'variable'};

    if (defined $self->{ERROR_MSG}) {
        $result .= '<br /><font color="#FF0000">' . $self->{ERROR_MSG} . '</font><br />';
    }

    $result .= <<ELEMENTHTML;

<p>How should the column be formatted?</p>

<table cellpadding='3'>
  <tr>
    <td align='right'><b>Width</b>:</td>
    <td align='left'><input type='text' name='$var.width' value='18' size='4'></td>
    <td align='left'>
      <select name='$var.widthunit'>
        <option>cm</option><option>in</option>
      </select>
    </td>
  </tr>
  <tr>
    <td align='right'><b>Height</b>:</td>
    <td align='left'><input type='text' name="$var.height" value="25.9" size='4'></td>
    <td align='left'>
      <select name='$var.heightunit'>
        <option>cm</option><option>in</option>
      </select>
    </td>
  </tr>
  <tr>
    <td align='right'><b>Left margin</b>:</td>
    <td align='left'><input type='text' name='$var.lmargin' value='-1.5' size='4'></td>
    <td align='left'>
      <select name='$var.lmarginunit'>
        <option>cm</option><option>in</option>
      </select>
    </td>
  </tr>
</table>

<p>Hint: Some instructors like to leave scratch space for the student by
making the width much smaller then the width of the page.</p>

ELEMENTHTML

    return $result;
}

# If the user didn't select 1 column, skip this state.
sub preprocess {
    my $self = shift;
    my $helper = Apache::lonhelper::getHelper();

    my $format = $helper->{VARS}->{$self->{'formatvar'}};
    if (substr($format, 2, 1) ne '1') {
        $helper->changeState($self->{NEXTSTATE});
    }
   
    return 1;
}

sub postprocess {
    my $self = shift;

    my $var = $self->{'variable'};
    my $helper = Apache::lonhelper->getHelper();
    my $width = $helper->{VARS}->{$var .'.width'} = $ENV{"form.${var}.width"}; 
    my $height = $helper->{VARS}->{$var .'.height'} = $ENV{"form.${var}.height"}; 
    my $lmargin = $helper->{VARS}->{$var .'.lmargin'} = $ENV{"form.${var}.lmargin"}; 
    $helper->{VARS}->{$var .'.widthunit'} = $ENV{"form.${var}.widthunit"}; 
    $helper->{VARS}->{$var .'.heightunit'} = $ENV{"form.${var}.heightunit"}; 
    $helper->{VARS}->{$var .'.lmarginunit'} = $ENV{"form.${var}.lmarginunit"}; 

    my $error = '';

    # /^-?[0-9]+(\.[0-9]*)?$/ -> optional minus, at least on digit, followed 
    # by an optional period, followed by digits, ending the string

    if ($width !~  /^-?[0-9]+(\.[0-9]*)?$/) {
        $error .= "Invalid width; please type only a number.<br />\n";
    }
    if ($height !~  /^-?[0-9]+(\.[0-9]*)?$/) {
        $error .= "Invalid height; please type only a number.<br />\n";
    }
    if ($lmargin !~  /^-?[0-9]+(\.[0-9]*)?$/) {
        $error .= "Invalid left margin; please type only a number.<br />\n";
    }

    if (!$error) {
        Apache::lonhelper::getHelper()->changeState($self->{NEXTSTATE});
        return 1;
    } else {
        $self->{ERROR_MSG} = $error;
        return 0;
    }
}



__END__

